import re

from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    """Implement automotive provider for ``tr_TR`` locale.

    Sources:

    - https://en.wikipedia.org/wiki/Vehicle_registration_plates_of_Turkey
    """

    license_formats = (
        "## ? ####",
        "## ? #####",
        "## ?? ###",
        "## ?? ####",
        "## ??? ##",
        "## ??? ###",
    )
    ascii_uppercase_turkish = "ABCDEFGHIJKLMNOPRSTUVYZ"

    def license_plate(self) -> str:
        """Generate a license plate."""
        temp = re.sub(
            r"\?",
            lambda x: self.random_element(self.ascii_uppercase_turkish),
            self.random_element(self.license_formats),
        )
        temp = temp.replace("##", "{:02d}", 1)
        temp = temp.format(self.random_element(range(1, 82)))
        return self.numerify(temp)



from .. import Provider as PersonProvider


class Provider(PersonProvider):

    formats_male = (
        "{{first_name_male}} {{last_name}}",
        "{{first_name_male}} {{last_name}}",
        "{{first_name_male}} {{last_name}}",
        "{{first_name_male}} {{last_name}}",
        "{{first_name_male}} {{last_name}}-{{last_name}}",
        "{{prefix_male}} {{first_name_male}} {{last_name}}",
    )

    formats_female = (
        "{{first_name_female}} {{last_name}}",
        "{{first_name_female}} {{last_name}}",
        "{{first_name_female}} {{last_name}}",
        "{{first_name_female}} {{last_name}}",
        "{{first_name_female}} {{last_name}}-{{last_name}}",
        "{{prefix_female}} {{first_name_female}} {{last_name}}",
    )

    formats = formats_male + formats_female

    # source: https://en.wikipedia.org/w/index.php?title=Category:Italian_masculine_given_names
    first_names_male = (
        "Achille",
        "Adamo",
        "Adelmo",
        "Adriano",
        "Agnolo",
        "Agostino",
        "Alberico",
        "Alberto",
        "Alderano",
        "Aldo",
        "Alessandro",
        "Alessio",
        "Alfio",
        "Alfredo",
        "Alphons",
        "Amadeo",
        "Amedeo",
        "Amico",
        "Amleto",
        "Angelo",
        "Annibale",
        "Ansaldo",
        "Antonello",
        "Antonino",
        "Antonio",
        "Armando",
        "Arnaldo",
        "Arnulfo",
        "Arsenio",
        "Arturo",
        "Atenulf",
        "Augusto",
        "Azeglio",
        "Baccio",
        "Baldassare",
        "Bartolomeo",
        "Benedetto",
        "Benito",
        "Benvenuto",
        "Beppe",
        "Bernardo",
        "Biagio",
        "Bruno",
        "Calcedonio",
        "Calogero",
        "Camillo",
        "Carlo",
        "Carmelo",
        "Cesare",
        "Cipriano",
        "Cirillo",
        "Ciro",
        "Claudio",
        "Coluccio",
        "Coriolano",
        "Corrado",
        "Costantino",
        "Costanzo",
        "Damiano",
        "Daniele",
        "Danilo",
        "Dante",
        "Dario",
        "Delfino",
        "Dino",
        "Dionigi",
        "Domenico",
        "Donatello",
        "Donato",
        "Durante",
        "Edoardo",
        "Elladio",
        "Elmo",
        "Emilio",
        "Ennio",
        "Enrico",
        "Enzio",
        "Enzo",
        "Eraldo",
        "Ermanno",
        "Ermenegildo",
        "Ermes",
        "Ernesto",
        "Ettore",
        "Ezio",
        "Fabio",
        "Fabrizio",
        "Fausto",
        "Fedele",
        "Federico",
        "Federigo",
        "Ferdinando",
        "Filippo",
        "Fiorenzo",
        "Fiorino",
        "Flavio",
        "Francesco",
        "Franco",
        "Fredo",
        "Fulvio",
        "Gabriele",
        "Gaetano",
        "Galasso",
        "Gaspare",
        "Gastone",
        "Geronimo",
        "Giacinto",
        "Giacobbe",
        "Giacomo",
        "Giampaolo",
        "Giampiero",
        "Gian",
        "Giancarlo",
        "Gianfrancesco",
        "Gianfranco",
        "Gianluca",
        "Gianluigi",
        "Gianmarco",
        "Gianni",
        "Gianpaolo",
        "Gianpietro",
        "Gilberto",
        "Gino",
        "Gioacchino",
        "Gioachino",
        "Gioele",
        "Gioffre",
        "Gionata",
        "Giorgio",
        "Giosuè",
        "Giovanni",
        "Girolamo",
        "Giuliano",
        "Giulio",
        "Giuseppe",
        "Giustino",
        "Goffredo",
        "Graziano",
        "Greco",
        "Guarino",
        "Guglielmo",
        "Guido",
        "Gustavo",
        "Hugo",
        "Ignazio",
        "Ippazio",
        "Ivan",
        "Ivo",
        "Jacopo",
        "Lamberto",
        "Lando",
        "Laureano",
        "Lazzaro",
        "Leonardo",
        "Leone",
        "Leopoldo",
        "Liberto",
        "Livio",
        "Lodovico",
        "Lorenzo",
        "Luca",
        "Luchino",
        "Luciano",
        "Lucio",
        "Ludovico",
        "Luigi",
        "Manuel",
        "Marcantonio",
        "Marcello",
        "Marco",
        "Mariano",
        "Mario",
        "Martino",
        "Martino",
        "Massimiliano",
        "Massimo",
        "Matteo",
        "Mattia",
        "Maurilio",
        "Maurizio",
        "Mauro",
        "Michelangelo",
        "Michele",
        "Micheletto",
        "Michelotto",
        "Milo",
        "Mirco",
        "Mirko",
        "Nanni",
        "Napoleone",
        "Niccolò",
        "Nico",
        "Nicola",
        "Nicolò",
        "Nino",
        "Orazio",
        "Oreste",
        "Orlando",
        "Osvaldo",
        "Ottavio",
        "Ottone",
        "Panfilo",
        "Paolo",
        "Paride",
        "Pasqual",
        "Pasquale",
        "Patrizio",
        "Pellegrino",
        "Pier",
        "Pierangelo",
        "Piergiorgio",
        "Piergiuseppe",
        "Pierluigi",
        "Piermaria",
        "Piero",
        "Pierpaolo",
        "Piersanti",
        "Pietro",
        "Pompeo",
        "Pomponio",
        "Puccio",
        "Raffaele",
        "Raffaellino",
        "Raffaello",
        "Raimondo",
        "Ranieri",
        "Rembrandt",
        "Renzo",
        "Riccardo",
        "Ricciotti",
        "Roberto",
        "Rocco",
        "Rodolfo",
        "Rolando",
        "Roman",
        "Romeo",
        "Romolo",
        "Ronaldo",
        "Rosario",
        "Ruggero",
        "Ruggiero",
        "Sabatino",
        "Salvatore",
        "Salvi",
        "Sandro",
        "Sante",
        "Santino",
        "Saverio",
        "Sebastiano",
        "Sergius",
        "Severino",
        "Silvestro",
        "Silvio",
        "Simone",
        "Stefano",
        "Telemaco",
        "Temistocle",
        "Tiziano",
        "Toni",
        "Tonino",
        "Torquato",
        "Tullio",
        "Ubaldo",
        "Uberto",
        "Ugo",
        "Ugolino",
        "Umberto",
        "Valerio",
        "Venancio",
        "Vincentio",
        "Vincenzo",
        "Virgilio",
        "Vito",
        "Vittorio",
    )
    # source: https://en.wikipedia.org/wiki/Category:Italian_feminine_given_names
    first_names_female = (
        "Adelasia",
        "Adele",
        "Adriana",
        "Alessandra",
        "Alessia",
        "Alina",
        "Allegra",
        "Amalia",
        "Amanda",
        "Angelica",
        "Angelina",
        "Anita",
        "Annalisa",
        "Annamaria",
        "Annetta",
        "Annunziata",
        "Antonella",
        "Antonia",
        "Antonietta",
        "Antonina",
        "Aria",
        "Aurora",
        "Barbara",
        "Beatrice",
        "Berenice",
        "Bettina",
        "Bianca",
        "Bianca",
        "Camilla",
        "Carla",
        "Carolina",
        "Cassandra",
        "Caterina",
        "Cecilia",
        "Chiara",
        "Claudia",
        "Clelia",
        "Concetta",
        "Cristina",
        "Daria",
        "Diana",
        "Dina",
        "Dolores",
        "Donatella",
        "Donna",
        "Eleanora",
        "Elena",
        "Eliana",
        "Elisa",
        "Elvira",
        "Emma",
        "Erika",
        "Etta",
        "Eugenia",
        "Eva",
        "Evangelista",
        "Fabia",
        "Fabrizia",
        "Federica",
        "Fernanda",
        "Fiamma",
        "Filippa",
        "Flavia",
        "Flora",
        "Fortunata",
        "Francesca",
        "Gabriella",
        "Gelsomina",
        "Gemma",
        "Germana",
        "Giada",
        "Gianna",
        "Giorgia",
        "Giovanna",
        "Giulia",
        "Giuliana",
        "Giulietta",
        "Giuseppina",
        "Gloria",
        "Graziella",
        "Greca",
        "Griselda",
        "Ida",
        "Ilaria",
        "Imelda",
        "Iolanda",
        "Irma",
        "Isa",
        "Isabella",
        "Jolanda",
        "Lara",
        "Laura",
        "Lauretta",
        "Letizia",
        "Liana",
        "Licia",
        "Lidia",
        "Liliana",
        "Lilla",
        "Lina",
        "Lisa",
        "Livia",
        "Lolita",
        "Loredana",
        "Loretta",
        "Lucia",
        "Luciana",
        "Lucrezia",
        "Ludovica",
        "Luigina",
        "Luisa",
        "Marcella",
        "Margherita",
        "Maria",
        "Maria",
        "Maria",
        "Mariana",
        "Marina",
        "Marisa",
        "Marissa",
        "Marta",
        "Martina",
        "Matilda",
        "Maura",
        "Melania",
        "Melina",
        "Melissa",
        "Mercedes",
        "Michela",
        "Milena",
        "Monica",
        "Morena",
        "Nadia",
        "Natalia",
        "Nedda",
        "Nicoletta",
        "Nina",
        "Ninetta",
        "Olga",
        "Ornella",
        "Paloma",
        "Paola",
        "Paoletta",
        "Patrizia",
        "Paulina",
        "Pierina",
        "Pina",
        "Priscilla",
        "Raffaella",
        "Ramona",
        "Renata",
        "Rita",
        "Roberta",
        "Romana",
        "Romina",
        "Rosa",
        "Rosalia",
        "Rosaria",
        "Rosina",
        "Rossana",
        "Sandra",
        "Serafina",
        "Serena",
        "Silvia",
        "Simonetta",
        "Sole",
        "Sonia",
        "Sophia",
        "Stefani",
        "Stefania",
        "Stella",
        "Susanna",
        "Sylvia",
        "Tatiana",
        "Teresa",
        "Tina",
        "Tiziana",
        "Tonia",
        "Valentina",
        "Valeria",
        "Vanessa",
        "Veronica",
        "Victoria",
        "Vincenza",
        "Virginia",
        "Viridiana",
        "Vittoria",
        "Zaira",
    )

    first_names = first_names_male + first_names_female

    # source: https://en.wiktionary.org/w/index.php?title=Category:Italian_surnames
    last_names = (
        "Abatantuono",
        "Abate",
        "Abba",
        "Abbagnale",
        "Accardo",
        "Acerbi",
        "Adinolfi",
        "Agazzi",
        "Agnesi",
        "Agostinelli",
        "Agostini",
        "Ajello",
        "Albertini",
        "Alboni",
        "Aldobrandi",
        "Alfieri",
        "Alfonsi",
        "Alighieri",
        "Almagià",
        "Aloisio",
        "Alonzi",
        "Altera",
        "Amaldi",
        "Amato",
        "Ammaniti",
        "Anastasi",
        "Andreotti",
        "Andreozzi",
        "Angeli",
        "Angiolello",
        "Anguillara",
        "Anguissola",
        "Anichini",
        "Antelami",
        "Antonacci",
        "Antonelli",
        "Antonello",
        "Antonetti",
        "Antonini",
        "Antonioni",
        "Antonucci",
        "Aporti",
        "Argan",
        "Argentero",
        "Argenti",
        "Argento",
        "Argurio",
        "Ariasso",
        "Ariosto",
        "Armani",
        "Armellini",
        "Asmundo",
        "Asprucci",
        "Aulenti",
        "Avogadro",
        "Babati",
        "Babato",
        "Babbo",
        "Bacosi",
        "Badoer",
        "Badoglio",
        "Baggio",
        "Baglioni",
        "Bajamonti",
        "Bajardi",
        "Balbi",
        "Balbo",
        "Balla",
        "Balotelli",
        "Bandello",
        "Baracca",
        "Barbarigo",
        "Barberini",
        "Barcaccia",
        "Barcella",
        "Barese",
        "Baresi",
        "Barillaro",
        "Baroffio",
        "Barozzi",
        "Barracco",
        "Barsanti",
        "Bartoli",
        "Barzini",
        "Basadonna",
        "Bassi",
        "Basso",
        "Bataglia",
        "Battaglia",
        "Battelli",
        "Battisti",
        "Bazzi",
        "Beccaria",
        "Beccheria",
        "Beffa",
        "Belletini",
        "Bellini",
        "Bellocchio",
        "Bellucci",
        "Bellò",
        "Bembo",
        "Benedetti",
        "Benigni",
        "Benussi",
        "Berengario",
        "Bergoglio",
        "Berlusconi",
        "Bernardi",
        "Bernardini",
        "Bernetti",
        "Bernini",
        "Berrè",
        "Bersani",
        "Bertoli",
        "Bertolucci",
        "Bertoni",
        "Bettin",
        "Bettoni",
        "Bevilacqua",
        "Biagi",
        "Biagiotti",
        "Bianchi",
        "Bianchini",
        "Bignami",
        "Bignardi",
        "Binaghi",
        "Bixio",
        "Blasi",
        "Boaga",
        "Bocca",
        "Boccaccio",
        "Boccherini",
        "Boccioni",
        "Bocelli",
        "Bodoni",
        "Boezio",
        "Boiardo",
        "Boitani",
        "Boito",
        "Boldù",
        "Bombieri",
        "Bompiani",
        "Bonanno",
        "Bonatti",
        "Bonaventura",
        "Bondumier",
        "Bongiorno",
        "Bonino",
        "Bonolis",
        "Bonomo",
        "Borghese",
        "Borgia",
        "Borrani",
        "Borromeo",
        "Borromini",
        "Borroni",
        "Borsellino",
        "Borsiere",
        "Borzomì",
        "Bosio",
        "Bossi",
        "Bosurgi",
        "Botta",
        "Bottaro",
        "Botticelli",
        "Bottigliero",
        "Bova",
        "Bragadin",
        "Bragaglia",
        "Bramante",
        "Brambilla",
        "Brancaccio",
        "Branciforte",
        "Brenna",
        "Bresciani",
        "Briccialdi",
        "Brichese",
        "Broggini",
        "Broschi",
        "Brugnaro",
        "Brunelleschi",
        "Brunello",
        "Bruno",
        "Bruscantini",
        "Bulzoni",
        "Buonauro",
        "Burcardo",
        "Buscetta",
        "Busoni",
        "Cabibbo",
        "Caboto",
        "Cabrini",
        "Caccianemico",
        "Caccioppoli",
        "Cadorna",
        "Caetani",
        "Cafarchia",
        "Caffarelli",
        "Cagnin",
        "Cagnotto",
        "Cainero",
        "Caironi",
        "Calarco",
        "Calbo",
        "Calgari",
        "Callegari",
        "Callegaro",
        "Calvo",
        "Camanni",
        "Camicione",
        "Camilleri",
        "Camiscione",
        "Cammarata",
        "Campanella",
        "Campano",
        "Campise",
        "Camuccini",
        "Canali",
        "Canetta",
        "Canevascini",
        "Canil",
        "Cannizzaro",
        "Canova",
        "Cantimori",
        "Capecchi",
        "Capone",
        "Cappelli",
        "Capuana",
        "Caracciolo",
        "Cardano",
        "Carducci",
        "Carfagna",
        "Carli",
        "Carnera",
        "Carocci",
        "Carosone",
        "Carpaccio",
        "Carriera",
        "Carullo",
        "Caruso",
        "Casadei",
        "Casagrande",
        "Casale",
        "Casaleggio",
        "Casalodi",
        "Casarin",
        "Casellati",
        "Casini",
        "Cassarà",
        "Castelli",
        "Castellitto",
        "Castiglione",
        "Castioni",
        "Catalano",
        "Catenazzi",
        "Cattaneo",
        "Cavalcanti",
        "Cavanna",
        "Ceci",
        "Celentano",
        "Cendron",
        "Ceravolo",
        "Ceri",
        "Cerquiglini",
        "Cerutti",
        "Cesaroni",
        "Cesarotti",
        "Ceschi",
        "Chechi",
        "Cheda",
        "Cherubini",
        "Chiappetta",
        "Chiaramonte",
        "Chiesa",
        "Chigi",
        "Chindamo",
        "Chinnici",
        "Chittolini",
        "Ciampi",
        "Cianciolo",
        "Ciani",
        "Cibin",
        "Cicala",
        "Cicilia",
        "Cignaroli",
        "Cilea",
        "Cilibrasi",
        "Cimarosa",
        "Cimini",
        "Cipolla",
        "Civaschi",
        "Coardi",
        "Cocci",
        "Cociarelli",
        "Colletti",
        "Collina",
        "Collodi",
        "Columbo",
        "Combi",
        "Comboni",
        "Comencini",
        "Comeriato",
        "Comisso",
        "Comolli",
        "Condoleo",
        "Contarini",
        "Conte",
        "Conti",
        "Contrafatto",
        "Coppola",
        "Corbo",
        "Corcos",
        "Corradi",
        "Correr",
        "Cortese",
        "Cossiga",
        "Costalonga",
        "Costanzi",
        "Cremonesi",
        "Crespi",
        "Crisafulli",
        "Crispi",
        "Cristoforetti",
        "Cuda",
        "Cugia",
        "Cundari",
        "Cuomo",
        "Curatoli",
        "Curci",
        "Curiel",
        "Cusano",
        "Cutrufo",
        "Cutuli",
        "Cuzzocrea",
        "Dalla",
        "Dallapé",
        "Dallara",
        "Dandolo",
        "Deledda",
        "Delle",
        "Dellucci",
        "Depero",
        "Desio",
        "Detti",
        "Dibiasi",
        "Disdero",
        "Doglioni",
        "Donarelli",
        "Donati",
        "Donatoni",
        "Donini",
        "Donà",
        "Doria",
        "Dossetti",
        "Dossi",
        "Dovara",
        "Draghi",
        "Druso",
        "Dulbecco",
        "Duodo",
        "Durante",
        "Duse",
        "Eco",
        "Einaudi",
        "Emanuelli",
        "Emo",
        "Endrizzi",
        "Errani",
        "Errigo",
        "Esposito",
        "Fabbri",
        "Fabrizi",
        "Faggiani",
        "Fagiani",
        "Fagotto",
        "Falcone",
        "Falier",
        "Fallaci",
        "Falloppio",
        "Fantini",
        "Fantoni",
        "Fantozzi",
        "Fanucci",
        "Faranda",
        "Farina",
        "Farinelli",
        "Farnese",
        "Fattori",
        "Faugno",
        "Favata",
        "Federici",
        "Federico",
        "Fermi",
        "Ferrabosco",
        "Ferragamo",
        "Ferragni",
        "Ferrante",
        "Ferrara",
        "Ferrari",
        "Ferraris",
        "Ferrata",
        "Ferrazzi",
        "Ferretti",
        "Ferrucci",
        "Fibonacci",
        "Ficino",
        "Fieramosca",
        "Filangieri",
        "Filippelli",
        "Filippini",
        "Filogamo",
        "Filzi",
        "Finetti",
        "Finotto",
        "Finzi",
        "Fioravanti",
        "Fiorucci",
        "Fischetti",
        "Fittipaldi",
        "Flaiano",
        "Florio",
        "Fo",
        "Foa",
        "Foconi",
        "Fogazzaro",
        "Foletti",
        "Folliero",
        "Fornaciari",
        "Forza",
        "Foscari",
        "Foà",
        "Fracci",
        "Franceschi",
        "Franscini",
        "Franzese",
        "Frescobaldi",
        "Fusani",
        "Fuseli",
        "Gabba",
        "Gabbana",
        "Gabrieli",
        "Gadda",
        "Gaggini",
        "Gagliano",
        "Gagliardi",
        "Gaiatto",
        "Gaito",
        "Galeati",
        "Galiazzo",
        "Galilei",
        "Galtarossa",
        "Galuppi",
        "Galvani",
        "Gangemi",
        "Gargallo",
        "Garibaldi",
        "Garobbio",
        "Garozzo",
        "Garrone",
        "Garzoni",
        "Gasperi",
        "Gatto",
        "Gelli",
        "Gemito",
        "Gentileschi",
        "Gentili",
        "Gentilini",
        "Geraci",
        "Germano",
        "Giacconi",
        "Giacometti",
        "Giammusso",
        "Gianetti",
        "Gianinazzi",
        "Giannelli",
        "Giannetti",
        "Giannini",
        "Giannone",
        "Giannotti",
        "Giannuzzi",
        "Gianvecchio",
        "Gibilisco",
        "Gigli",
        "Gilardoni",
        "Ginese",
        "Ginesio",
        "Gioberti",
        "Giolitti",
        "Giorgetti",
        "Giovine",
        "Giradello",
        "Giulietti",
        "Giunti",
        "Giusti",
        "Goldoni",
        "Goldstein",
        "Golgi",
        "Golino",
        "Gonzaga",
        "Gori",
        "Gottardi",
        "Gotti",
        "Govoni",
        "Gozzano",
        "Gozzi",
        "Gradenigo",
        "Gramsci",
        "Granatelli",
        "Grassi",
        "Grasso",
        "Gravina",
        "Greco",
        "Greggio",
        "Gregori",
        "Gregorio",
        "Gremese",
        "Grifeo",
        "Grimani",
        "Grisoni",
        "Gritti",
        "Grossi",
        "Gualandi",
        "Gualtieri",
        "Guarana",
        "Guarato",
        "Guariento",
        "Guarneri",
        "Gucci",
        "Guglielmi",
        "Guicciardini",
        "Guidone",
        "Guidotti",
        "Guinizzelli",
        "Gullotta",
        "Gulotta",
        "Gussoni",
        "Iacobucci",
        "Iacovelli",
        "Iadanza",
        "Iannelli",
        "Iannotti",
        "Iannucci",
        "Iannuzzi",
        "Impastato",
        "Infantino",
        "Innocenti",
        "Interiano",
        "Interminei",
        "Interminelli",
        "Inzaghi",
        "Ioppi",
        "Jacuzzi",
        "Jilani",
        "Jovinelli",
        "Juvara",
        "Lamborghini",
        "Lancisi",
        "Lanfranchi",
        "Lattuada",
        "Leblanc",
        "Legnante",
        "Leonardi",
        "Leoncavallo",
        "Leone",
        "Leonetti",
        "Leopardi",
        "Lercari",
        "Lerner",
        "Letta",
        "Lettiere",
        "Ligorio",
        "Liguori",
        "Lippomano",
        "Littizzetto",
        "Liverotti",
        "Lollobrigida",
        "Lombardi",
        "Lombardo",
        "Lombroso",
        "Longhena",
        "Lopresti",
        "Loredan",
        "Lovato",
        "Lucarelli",
        "Lucchesi",
        "Lucciano",
        "Luciani",
        "Ludovisi",
        "Luna",
        "Lupo",
        "Luria",
        "Lussu",
        "Luxardo",
        "Luzi",
        "Maccanelli",
        "Maderna",
        "Maderno",
        "Maffei",
        "Maggioli",
        "Maglio",
        "Magnani",
        "Magrassi",
        "Majewski",
        "Majorana",
        "Malacarne",
        "Malaparte",
        "Malatesta",
        "Malenchini",
        "Malipiero",
        "Malpighi",
        "Manacorda",
        "Mancini",
        "Mannoia",
        "Manolesso",
        "Mantegazza",
        "Mantegna",
        "Manunta",
        "Manzoni",
        "Marangoni",
        "Marazzi",
        "Marcacci",
        "Marconi",
        "Marenzio",
        "Marinetti",
        "Marini",
        "Marino",
        "Marrone",
        "Marsili",
        "Martinelli",
        "Martucci",
        "Marzorati",
        "Mascagni",
        "Mascheroni",
        "Maspero",
        "Mastandrea",
        "Mastroianni",
        "Mattarella",
        "Matteotti",
        "Mazzacurati",
        "Mazzanti",
        "Mazzeo",
        "Mazzi",
        "Mazzini",
        "Mazzocchi",
        "Medici",
        "Mengolo",
        "Mennea",
        "Mercadante",
        "Mercalli",
        "Mercantini",
        "Mercati",
        "Merisi",
        "Metella",
        "Meucci",
        "Mezzetta",
        "Micca",
        "Michelangeli",
        "Micheletti",
        "Migliaccio",
        "Milanesi",
        "Mimun",
        "Miniati",
        "Missoni",
        "Moccia",
        "Mocenigo",
        "Modiano",
        "Modigliani",
        "Modugno",
        "Mogherini",
        "Molesini",
        "Monaco",
        "Mondadori",
        "Mondaini",
        "Monduzzi",
        "Moneta",
        "Monicelli",
        "Montalcini",
        "Montalti",
        "Montanari",
        "Montanariello",
        "Montanelli",
        "Monte",
        "Montecchi",
        "Montesano",
        "Montessori",
        "Monteverdi",
        "Monti",
        "Morabito",
        "Morandi",
        "Morandini",
        "Morellato",
        "Moresi",
        "Moretti",
        "Morgagni",
        "Morlacchi",
        "Morosini",
        "Morpurgo",
        "Morricone",
        "Morrocco",
        "Mortati",
        "Morucci",
        "Moschino",
        "Mozart",
        "Munari",
        "Muratori",
        "Murialdo",
        "Murri",
        "Musatti",
        "Muti",
        "Naccari",
        "Nadi",
        "Napolitano",
        "Natta",
        "Navarria",
        "Navone",
        "Necci",
        "Nibali",
        "Nicoletti",
        "Nicolini",
        "Nicolucci",
        "Nievo",
        "Niggli",
        "Niscoromni",
        "Nitti",
        "Nitto",
        "Nolcini",
        "Nonis",
        "Norbiato",
        "Nordio",
        "Nosiglia",
        "Notarbartolo",
        "Novaro",
        "Nugnes",
        "Odescalchi",
        "Offredi",
        "Oliboni",
        "Olivetti",
        "Omma",
        "Onio",
        "Onisto",
        "Opizzi",
        "Orengo",
        "Orlando",
        "Orsini",
        "Ortese",
        "Ortolani",
        "Oscuro",
        "Ossani",
        "Ossola",
        "Ostinelli",
        "Ottino",
        "Ovadia",
        "Pace",
        "Pacelli",
        "Pacetti",
        "Pacillo",
        "Pacomio",
        "Padovano",
        "Paganini",
        "Pagliaro",
        "Pagnotto",
        "Palazzo",
        "Palladio",
        "Palmisano",
        "Palombi",
        "Paltrinieri",
        "Palumbo",
        "Panatta",
        "Panicucci",
        "Panzera",
        "Paoletti",
        "Paolini",
        "Paolucci",
        "Papafava",
        "Papetti",
        "Pareto",
        "Parini",
        "Parisi",
        "Parmitano",
        "Parpinel",
        "Parri",
        "Paruta",
        "Pascarella",
        "Pasolini",
        "Pasqua",
        "Passalacqua",
        "Pastine",
        "Pausini",
        "Pavanello",
        "Pavarotti",
        "Pavone",
        "Peano",
        "Pederiva",
        "Pedersoli",
        "Pedrazzini",
        "Pedroni",
        "Pellegrini",
        "Pelli",
        "Pellico",
        "Pennetta",
        "Pepe",
        "Peranda",
        "Pergolesi",
        "Perini",
        "Perozzo",
        "Persico",
        "Pertile",
        "Pertini",
        "Peruzzi",
        "Petralli",
        "Petrassi",
        "Petrocelli",
        "Petrucci",
        "Petrucelli",
        "Petruzzi",
        "Pezzali",
        "Piacentini",
        "Piane",
        "Piazzi",
        "Piccinni",
        "Piccio",
        "Pietrangeli",
        "Pigafetta",
        "Pignatti",
        "Pinamonte",
        "Pincherle",
        "Pininfarina",
        "Piovani",
        "Pirandello",
        "Pirelli",
        "Pisacane",
        "Pisani",
        "Pisano",
        "Pisaroni",
        "Pistoletto",
        "Pizzamano",
        "Pizzetti",
        "Pizziol",
        "Pizzo",
        "Platini",
        "Poerio",
        "Polani",
        "Polesel",
        "Polizzi",
        "Pometta",
        "Pontecorvo",
        "Ponti",
        "Porcellato",
        "Porzio",
        "Pozzecco",
        "Prada",
        "Praga",
        "Pratesi",
        "Prati",
        "Priuli",
        "Procacci",
        "Prodi",
        "Proietti",
        "Pucci",
        "Puccini",
        "Pugliese",
        "Puglisi",
        "Pulci",
        "Quasimodo",
        "Querini",
        "Raimondi",
        "Ramazzotti",
        "Randazzo",
        "Rapisardi",
        "Rastelli",
        "Raurica",
        "Ravaglioli",
        "Redi",
        "Regge",
        "Renault",
        "Renier",
        "Rensi",
        "Renzi",
        "Respighi",
        "Riccardi",
        "Riccati",
        "Ricci",
        "Ricciardi",
        "Ricolfi",
        "Rienzo",
        "Righi",
        "Rinaldi",
        "Rismondo",
        "Ritacca",
        "Rizzo",
        "Rizzoli",
        "Rocca",
        "Roccabonella",
        "Roero",
        "Romagnoli",
        "Romano",
        "Romiti",
        "Roncalli",
        "Rosiello",
        "Rosmini",
        "Rosselli",
        "Rossellini",
        "Rossetti",
        "Rossi",
        "Rossini",
        "Roth",
        "Rubbia",
        "Ruberto",
        "Ruffini",
        "Ruggeri",
        "Ruggieri",
        "Russo",
        "Rusticucci",
        "Sabatini",
        "Sabbatini",
        "Saffi",
        "Sagese",
        "Sagnelli",
        "Sagredo",
        "Salandra",
        "Salata",
        "Salgari",
        "Salieri",
        "Salvemini",
        "Salvini",
        "Salvo",
        "Samele",
        "Sandi",
        "Sanguineti",
        "Sansoni",
        "Santi",
        "Santorio",
        "Santoro",
        "Sanudo",
        "Saraceno",
        "Saracino",
        "Saragat",
        "Satriani",
        "Satta",
        "Sauli",
        "Sauro",
        "Savorgnan",
        "Sbarbaro",
        "Scaduto",
        "Scalera",
        "Scalfaro",
        "Scamarcio",
        "Scandone",
        "Scaramucci",
        "Scarfoglio",
        "Scarlatti",
        "Scarpa",
        "Scarpetta",
        "Scarponi",
        "Schiaparelli",
        "Schiavo",
        "Schiavone",
        "Schicchi",
        "Scialpi",
        "Scotti",
        "Scotto",
        "Seddio",
        "Segni",
        "Segrè",
        "Semitecolo",
        "Serao",
        "Serlupi",
        "Sermonti",
        "Serraglio",
        "Sforza",
        "Sgalambro",
        "Sgarbi",
        "Sibilia",
        "Siffredi",
        "Silvestri",
        "Simeoni",
        "Sinisi",
        "Sismondi",
        "Smirnoff",
        "Sobrero",
        "Soderini",
        "Soffici",
        "Sokolov",
        "Solari",
        "Solimena",
        "Sollima",
        "Sommaruga",
        "Sonnino",
        "Soprano",
        "Soranzo",
        "Sordi",
        "Sorrentino",
        "Spadafora",
        "Spallanzani",
        "Spanevello",
        "Speri",
        "Spinelli",
        "Spinola",
        "Squarcione",
        "Sraffa",
        "Staglieno",
        "Stefanelli",
        "Stein",
        "Stoppani",
        "Storladi",
        "Stradivari",
        "Strangio",
        "Stucchi",
        "Surian",
        "Tacchini",
        "Taccola",
        "Tafuri",
        "Tagliafierro",
        "Taliani",
        "Taliercio",
        "Tamborini",
        "Tamburello",
        "Tamburi",
        "Tamburini",
        "Tanzini",
        "Tarantini",
        "Tarantino",
        "Tarchetti",
        "Tartaglia",
        "Tartini",
        "Tasca",
        "Tasso",
        "Tassoni",
        "Tebaldi",
        "Tedesco",
        "Telesio",
        "Tencalla",
        "Terragni",
        "Tiepolo",
        "Tirabassi",
        "Togliatti",
        "Tognazzi",
        "Toldo",
        "Tolentino",
        "Tomaselli",
        "Tomasetti",
        "Tomasini",
        "Tomei",
        "Tommaseo",
        "Toninelli",
        "Tonisto",
        "Torlonia",
        "Tornatore",
        "Torricelli",
        "Toscani",
        "Toscanini",
        "Toselli",
        "Tosi",
        "Toso",
        "Tosto",
        "Totino",
        "Tozzi",
        "Tozzo",
        "Traetta",
        "Trapanese",
        "Trapani",
        "Travaglia",
        "Travaglio",
        "Traversa",
        "Travia",
        "Trebbi",
        "Treccani",
        "Tremonti",
        "Trentin",
        "Trentini",
        "Tresoldi",
        "Treves",
        "Trevisan",
        "Trevisani",
        "Trezzini",
        "Trillini",
        "Trincavelli",
        "Trobbiani",
        "Troisi",
        "Trombetta",
        "Tron",
        "Tropea",
        "Trotta",
        "Trupiano",
        "Trussardi",
        "Turati",
        "Turchetta",
        "Turchi",
        "Turci",
        "Turrini",
        "Tutino",
        "Tuzzolino",
        "Ubaldi",
        "Udinese",
        "Udinesi",
        "Ughi",
        "Ungaretti",
        "Valentino",
        "Valguarnera",
        "Valier",
        "Valmarana",
        "Vanvitelli",
        "Varano",
        "Vasari",
        "Vattimo",
        "Vecellio",
        "Vecoli",
        "Veltroni",
        "Vendetti",
        "Venditti",
        "Veneziano",
        "Venier",
        "Vento",
        "Venturi",
        "Vercelloni",
        "Verdi",
        "Verdone",
        "Verga",
        "Vergassola",
        "Vergerio",
        "Verri",
        "Versace",
        "Vespa",
        "Vespucci",
        "Vezzali",
        "Vianello",
        "Vidoni",
        "Vigliotti",
        "Vigorelli",
        "Villadicani",
        "Villarosa",
        "Viola",
        "Virgilio",
        "Visconti",
        "Visintini",
        "Vismara",
        "Vittadello",
        "Vitturi",
        "Vivaldi",
        "Viviani",
        "Volta",
        "Volterra",
        "Zabarella",
        "Zaccagnini",
        "Zaccardo",
        "Zacchia",
        "Zacco",
        "Zaguri",
        "Zamengo",
        "Zamorani",
        "Zampa",
        "Zanazzo",
        "Zanichelli",
        "Zanzi",
        "Zarlino",
        "Zecchini",
        "Zeffirelli",
        "Zetticci",
        "Ziani",
        "Zichichi",
        "Zito",
        "Zola",
        "Zoppetti",
        "Zoppetto",
    )

    prefixes_female = ("Dott.", "Sig.ra")
    prefixes_male = ("Dott.", "Sig.")

    prefixes = ("Dott.", "Sig.", "Sig.ra")



from typing import Tuple

from .. import Provider as CompanyProvider


class Provider(CompanyProvider):
    formats = (
        "{{last_name}} {{company_suffix}}",
        "{{last_name}} {{last_name}} {{company_suffix}}",
        "{{last_name}}",
        "{{last_name}}",
    )

    catch_phrase_formats = ("{{catch_phrase_noun}} {{catch_phrase_verb}} {{catch_phrase_attribute}}",)

    nouns = (
        "la sécurité",
        "le plaisir",
        "le confort",
        "la simplicité",
        "l'assurance",
        "l'art",
        "le pouvoir",
        "le droit",
        "la possibilité",
        "l'avantage",
        "la liberté",
    )

    verbs = (
        "de rouler",
        "d'avancer",
        "d'évoluer",
        "de changer",
        "d'innover",
        "de louer",
        "d'atteindre vos buts",
        "de concrétiser vos projets",
    )

    attributes = (
        "de manière efficace",
        "plus rapidement",
        "plus facilement",
        "plus simplement",
        "en toute tranquilité",
        "avant-tout",
        "autrement",
        "naturellement",
        "à la pointe",
        "sans soucis",
        "à l'état pur",
        "à sa source",
        "de manière sûre",
        "en toute sécurité",
    )

    company_suffixes: Tuple[str, ...] = (
        "SA",
        "S.A.",
        "SARL",
        "S.A.R.L.",
        "S.A.S.",
        "et Fils",
    )

    siren_format = "### ### ###"

    def catch_phrase_noun(self) -> str:
        """
        Returns a random catch phrase noun.
        """
        return self.random_element(self.nouns)

    def catch_phrase_attribute(self) -> str:
        """
        Returns a random catch phrase attribute.
        """
        return self.random_element(self.attributes)

    def catch_phrase_verb(self) -> str:
        """
        Returns a random catch phrase verb.
        """
        return self.random_element(self.verbs)

    def catch_phrase(self) -> str:
        """
        :example 'integrate extensible convergence'
        """
        catch_phrase = ""
        while True:

            pattern: str = self.random_element(self.catch_phrase_formats)
            catch_phrase = self.generator.parse(pattern)
            catch_phrase = catch_phrase[0].upper() + catch_phrase[1:]

            if self._is_catch_phrase_valid(catch_phrase):
                break

        return catch_phrase

    # An array containing string which should not appear twice in a catch phrase
    words_which_should_not_appear_twice = ("sécurité", "simpl")

    def _is_catch_phrase_valid(self, catch_phrase: str) -> bool:
        """
        Validates a french catch phrase.

        :param catch_phrase: The catch phrase to validate.
        """
        for word in self.words_which_should_not_appear_twice:
            # Fastest way to check if a piece of word does not appear twice.
            begin_pos = catch_phrase.find(word)
            end_pos = catch_phrase.find(word, begin_pos + 1)

            if begin_pos != -1 and begin_pos != end_pos:
                return False

        return True

    def siren(self) -> str:
        """
        Generates a siren number (9 digits).
        """
        return self.numerify(self.siren_format)

    def siret(self, max_sequential_digits: int = 2) -> str:
        """
        Generates a siret number (14 digits).
        It is in fact the result of the concatenation of a siren number (9 digits),
        a sequential number (4 digits) and a control number (1 digit) concatenation.
        If $max_sequential_digits is invalid, it is set to 2.
        :param max_sequential_digits The maximum number of digits for the sequential number (> 0 && <= 4).
        """
        if max_sequential_digits > 4 or max_sequential_digits <= 0:
            max_sequential_digits = 2

        sequential_number = str(self.random_number(max_sequential_digits)).zfill(4)
        return self.numerify(self.siren() + " " + sequential_number + "#")



from .. import Provider as InternetProvider


class Provider(InternetProvider):

    user_name_formats = (
        "{{last_name}}.{{first_name_female}}",
        "{{last_name}}.{{first_name_male}}",
        "{{first_name_female}}.{{last_name}}",
        "{{first_name_male}}.{{last_name}}",
        "{{first_name}}##",
    )

    email_formats = ("{{user_name}}@{{free_email_domain}}",)

    free_email_domains = (
        "gmail.com",
        "siol.net",
        "email.si",
        "volja.net",
    )

    uri_pages = (
        "index",
        "domov",
        "iskanje",
        "main",
        "novica",
        "homepage",
        "kategorija",
        "registracija",
        "login",
        "faq",
        "o-nas",
        "pogoji",
        "zasebnost",
        "avtor",
    )
    uri_paths = (
        "app",
        "main",
        "wp-content",
        "iskanje",
        "kategorija",
        "novica",
        "kategorije",
        "novice",
        "blog",
        "komentarji",
        "seznam",
    )
    uri_extensions = (
        ".html",
        ".html",
        ".html",
        ".htm",
        ".htm",
        ".php",
        ".php",
        ".jsp",
        ".asp",
    )

    tlds = ("si", "com")



import re

from string import ascii_uppercase

from .. import BaseProvider, ElementsType

localized = True


class Provider(BaseProvider):
    """Implement default automotive provider for Faker."""

    license_formats: ElementsType = ()

    def license_plate(self) -> str:
        """Generate a license plate."""
        temp = re.sub(
            r"\?",
            lambda x: self.random_element(ascii_uppercase),
            self.random_element(self.license_formats),
        )
        return self.numerify(temp)



from typing import Tuple

from ..es import Provider as PersonProvider


class Provider(PersonProvider):
    formats_male: Tuple[str, ...] = (
        "{{first_name_male}} {{last_name}} {{last_name}}",
        "{{first_name_male}} {{last_name}} {{last_name}}",
        "{{first_name_male}} {{last_name}} {{last_name}}",
        "{{first_name_male}} {{last_name}} {{last_name}}",
        "{{first_name_male}} {{last_name}} {{last_name}}",
        "{{first_name_male}} {{last_name}} {{last_name}}",
        "{{first_name_male}} {{last_name}}",
        "{{first_name_male}} {{prefix}} {{last_name}}",
        "{{first_name_male}} {{last_name}}-{{last_name}}",
        "{{first_name_male}} {{first_name_male}} {{last_name}} {{last_name}}",
    )

    formats_female: Tuple[str, ...] = (
        "{{first_name_female}} {{last_name}} {{last_name}}",
        "{{first_name_female}} {{last_name}} {{last_name}}",
        "{{first_name_female}} {{last_name}} {{last_name}}",
        "{{first_name_female}} {{last_name}} {{last_name}}",
        "{{first_name_female}} {{last_name}} {{last_name}}",
        "{{first_name_female}} {{last_name}} {{last_name}}",
        "{{first_name_female}} {{last_name}}",
        "{{first_name_female}} {{prefix}} {{last_name}}",
        "{{first_name_female}} {{last_name}}-{{last_name}}",
        "{{first_name_female}} {{first_name_female}} {{last_name}} {{last_name}}",
    )

    formats: Tuple[str, ...] = formats_male + formats_female

    # 477 male first names, alphabetically.
    # Source: Álvaro Mondéjar Rubio <mondejar1994@gmail.com>
    first_names_male: Tuple[str, ...] = (
        "Aarón",
        "Abel",
        "Abilio",
        "Abraham",
        "Adalberto",
        "Adelardo",
        "Adolfo",
        "Adrián",
        "Adán",
        "Agapito",
        "Agustín",
        "Aitor",
        "Albano",
        "Albert",
        "Alberto",
        "Albino",
        "Alcides",
        "Ale",
        "Alejandro",
        "Alejo",
        "Alex",
        "Alfonso",
        "Alfredo",
        "Alonso",
        "Amado",
        "Amador",
        "Amancio",
        "Amando",
        "Amaro",
        "Ambrosio",
        "Amor",
        "Américo",
        "Amílcar",
        "Anacleto",
        "Anastasio",
        "Andrés",
        "Andrés Felipe",
        "Angelino",
        "Anselmo",
        "Antonio",
        "Aníbal",
        "Apolinar",
        "Ariel",
        "Aristides",
        "Armando",
        "Arsenio",
        "Artemio",
        "Arturo",
        "Asdrubal",
        "Atilio",
        "Augusto",
        "Aureliano",
        "Aurelio",
        "Baldomero",
        "Balduino",
        "Baltasar",
        "Bartolomé",
        "Basilio",
        "Baudelio",
        "Bautista",
        "Benigno",
        "Benito",
        "Benjamín",
        "Bernabé",
        "Bernardino",
        "Bernardo",
        "Berto",
        "Blas",
        "Bonifacio",
        "Borja",
        "Bruno",
        "Buenaventura",
        "Calisto",
        "Calixto",
        "Camilo",
        "Candelario",
        "Carlito",
        "Carlos",
        "Carmelo",
        "Casemiro",
        "Cayetano",
        "Cebrián",
        "Cecilio",
        "Ceferino",
        "Celestino",
        "Celso",
        "Cesar",
        "Che",
        "Chema",
        "Chucho",
        "Chus",
        "Chuy",
        "Cipriano",
        "Ciriaco",
        "Cirino",
        "Ciro",
        "Ciríaco",
        "Claudio",
        "Clemente",
        "Cleto",
        "Clímaco",
        "Conrado",
        "Cornelio",
        "Cosme",
        "Cristian",
        "Cristian",
        "Cristóbal",
        "Cruz",
        "Curro",
        "Custodio",
        "Cándido",
        "César",
        "Damián",
        "Dan",
        "Dani",
        "Daniel",
        "Danilo",
        "Darío",
        "David",
        "Demetrio",
        "Desiderio",
        "Diego",
        "Dimas",
        "Dionisio",
        "Domingo",
        "Donato",
        "Duilio",
        "Edelmiro",
        "Edgardo",
        "Edmundo",
        "Edu",
        "Eduardo",
        "Efraín",
        "Eladio",
        "Eleuterio",
        "Eligio",
        "Eliseo",
        "Eloy",
        "Elpidio",
        "Elías",
        "Emigdio",
        "Emiliano",
        "Emilio",
        "Enrique",
        "Epifanio",
        "Erasmo",
        "Eric",
        "Ernesto",
        "Espiridión",
        "Esteban",
        "Eugenio",
        "Eusebio",
        "Eustaquio",
        "Eutimio",
        "Eutropio",
        "Evaristo",
        "Ezequiel",
        "Fabio",
        "Fabián",
        "Fabricio",
        "Faustino",
        "Fausto",
        "Federico",
        "Feliciano",
        "Felipe",
        "Felix",
        "Fermín",
        "Fernando",
        "Fidel",
        "Fito",
        "Flavio",
        "Florencio",
        "Florentino",
        "Fortunato",
        "Francisco",
        "Francisco Javier",
        "Francisco Jose",
        "Fulgencio",
        "Félix",
        "Gabino",
        "Gabriel",
        "Galo",
        "Gaspar",
        "Gastón",
        "Geraldo",
        "Gerardo",
        "Germán",
        "Gervasio",
        "Gerónimo",
        "Gil",
        "Gilberto",
        "Glauco",
        "Godofredo",
        "Gonzalo",
        "Goyo",
        "Graciano",
        "Gregorio",
        "Guadalupe",
        "Guillermo",
        "Guiomar",
        "Gustavo",
        "Haroldo",
        "Hector",
        "Heliodoro",
        "Heraclio",
        "Herberto",
        "Heriberto",
        "Hermenegildo",
        "Herminio",
        "Hernando",
        "Hernán",
        "Hilario",
        "Hipólito",
        "Horacio",
        "Hugo",
        "Humberto",
        "Héctor",
        "Ibán",
        "Ignacio",
        "Iker",
        "Ildefonso",
        "Inocencio",
        "Isaac",
        "Isaías",
        "Isidoro",
        "Isidro",
        "Ismael",
        "Iván",
        "Jacinto",
        "Jacobo",
        "Jafet",
        "Jaime",
        "Javi",
        "Javier",
        "Jenaro",
        "Jeremías",
        "Jerónimo",
        "Jesús",
        "Joan",
        "Joaquín",
        "Joel",
        "Jonatan",
        "Jordi",
        "Jordán",
        "Jorge",
        "Jose",
        "Jose Angel",
        "Jose Antonio",
        "Jose Carlos",
        "Jose Francisco",
        "Jose Ignacio",
        "Jose Luis",
        "Jose Manuel",
        "Jose Miguel",
        "Jose Ramón",
        "Josep",
        "Josué",
        "José",
        "José Antonio",
        "José Luis",
        "José Manuel",
        "José Mari",
        "José María",
        "José Ángel",
        "Juan",
        "Juan Antonio",
        "Juan Bautista",
        "Juan Carlos",
        "Juan Francisco",
        "Juan José",
        "Juan Luis",
        "Juan Manuel",
        "Juan Pablo",
        "Juanito",
        "Julio",
        "Julio César",
        "Julián",
        "Kike",
        "Lalo",
        "Leandro",
        "Leocadio",
        "Leonardo",
        "Leoncio",
        "Leonel",
        "Leopoldo",
        "León",
        "Lino",
        "Lisandro",
        "Lope",
        "Lorenzo",
        "Loreto",
        "Lucas",
        "Lucho",
        "Luciano",
        "Lucio",
        "Luis",
        "Luis Miguel",
        "Luis Ángel",
        "Lupe",
        "Luís",
        "Lázaro",
        "Macario",
        "Manolo",
        "Manu",
        "Manuel",
        "Marc",
        "Marcelino",
        "Marcelo",
        "Marcial",
        "Marciano",
        "Marcio",
        "Marco",
        "Marcos",
        "Mariano",
        "Marino",
        "Mario",
        "Martin",
        "Martín",
        "María",
        "Mateo",
        "Matías",
        "Mauricio",
        "Maxi",
        "Maximiano",
        "Maximiliano",
        "Maximino",
        "Melchor",
        "Miguel",
        "Miguel Ángel",
        "Modesto",
        "Mohamed",
        "Moisés",
        "Moreno",
        "Máximo",
        "Nacho",
        "Nacio",
        "Nando",
        "Narciso",
        "Natalio",
        "Natanael",
        "Nazaret",
        "Nazario",
        "Nicanor",
        "Nico",
        "Nicodemo",
        "Nicolás",
        "Nilo",
        "Norberto",
        "Noé",
        "Néstor",
        "Octavio",
        "Olegario",
        "Omar",
        "Onofre",
        "Osvaldo",
        "Ovidio",
        "Pablo",
        "Paco",
        "Pancho",
        "Pascual",
        "Pastor",
        "Patricio",
        "Paulino",
        "Pedro",
        "Pelayo",
        "Pepe",
        "Pepito",
        "Plinio",
        "Plácido",
        "Poncio",
        "Porfirio",
        "Primitivo",
        "Prudencio",
        "Pánfilo",
        "Pío",
        "Quique",
        "Quirino",
        "Rafa",
        "Rafael",
        "Raimundo",
        "Ramiro",
        "Ramón",
        "Raúl",
        "Reinaldo",
        "Remigio",
        "Renato",
        "René",
        "Reyes",
        "Reynaldo",
        "Ricardo",
        "Rico",
        "Roberto",
        "Rodolfo",
        "Rodrigo",
        "Rogelio",
        "Rolando",
        "Roldán",
        "Román",
        "Roque",
        "Rosario",
        "Rosendo",
        "Ruben",
        "Rubén",
        "Rufino",
        "Ruperto",
        "Ruy",
        "Régulo",
        "Rómulo",
        "Sabas",
        "Salomón",
        "Salvador",
        "Samu",
        "Samuel",
        "Sancho",
        "Sandalio",
        "Santiago",
        "Santos",
        "Saturnino",
        "Sebastian",
        "Sebastián",
        "Segismundo",
        "Sergio",
        "Seve",
        "Severiano",
        "Severino",
        "Severo",
        "Sigfrido",
        "Silvestre",
        "Silvio",
        "Simón",
        "Sosimo",
        "Tadeo",
        "Telmo",
        "Teo",
        "Teobaldo",
        "Teodoro",
        "Teodosio",
        "Teófilo",
        "Tiburcio",
        "Timoteo",
        "Tito",
        "Tomás",
        "Toni",
        "Toribio",
        "Toño",
        "Trinidad",
        "Tristán",
        "Ulises",
        "Urbano",
        "Valentín",
        "Valerio",
        "Valero",
        "Vasco",
        "Venceslás",
        "Vicente",
        "Victor",
        "Victor Manuel",
        "Victoriano",
        "Victorino",
        "Vidal",
        "Vinicio",
        "Virgilio",
        "Vito",
        "Víctor",
        "Wilfredo",
        "Wálter",
        "Xavier",
        "Yago",
        "Zacarías",
        "Álvaro",
        "Ángel",
        "Édgar",
        "Íñigo",
        "Óscar",
    )

    # 477 female first names, alphabetically.
    # Source: Álvaro Mondéjar Rubio <mondejar1994@gmail.com>
    first_names_female: Tuple[str, ...] = (
        "Abigaíl",
        "Abril",
        "Adela",
        "Adelaida",
        "Adelia",
        "Adelina",
        "Adora",
        "Adoración",
        "Adriana",
        "Agustina",
        "Ainara",
        "Ainoa",
        "Aitana",
        "Alba",
        "Albina",
        "Ale",
        "Alejandra",
        "Alexandra",
        "Alicia",
        "Alma",
        "Almudena",
        "Alondra",
        "Amada",
        "Amalia",
        "Amanda",
        "Amarilis",
        "Amaya",
        "Amelia",
        "Amor",
        "Amparo",
        "América",
        "Ana",
        "Ana Belén",
        "Ana Sofía",
        "Anabel",
        "Anastasia",
        "Andrea",
        "Angelina",
        "Angelita",
        "Angélica",
        "Ani",
        "Anita",
        "Anna",
        "Anselma",
        "Antonia",
        "Anunciación",
        "Apolonia",
        "Araceli",
        "Arcelia",
        "Ariadna",
        "Ariel",
        "Armida",
        "Aroa",
        "Aránzazu",
        "Ascensión",
        "Asunción",
        "Aura",
        "Aurelia",
        "Aurora",
        "Azahar",
        "Azahara",
        "Azeneth",
        "Azucena",
        "Beatriz",
        "Begoña",
        "Belen",
        "Belén",
        "Benigna",
        "Benita",
        "Bernarda",
        "Bernardita",
        "Berta",
        "Bibiana",
        "Bienvenida",
        "Blanca",
        "Brunilda",
        "Brígida",
        "Bárbara",
        "Calista",
        "Calixta",
        "Camila",
        "Candela",
        "Candelaria",
        "Candelas",
        "Caridad",
        "Carina",
        "Carla",
        "Carlota",
        "Carmela",
        "Carmelita",
        "Carmen",
        "Carmina",
        "Carolina",
        "Casandra",
        "Catalina",
        "Cayetana",
        "Cecilia",
        "Celestina",
        "Celia",
        "Charo",
        "Chelo",
        "Chita",
        "Chus",
        "Cintia",
        "Clara",
        "Clarisa",
        "Claudia",
        "Clementina",
        "Cloe",
        "Clotilde",
        "Concepción",
        "Concha",
        "Constanza",
        "Consuela",
        "Consuelo",
        "Coral",
        "Corona",
        "Crescencia",
        "Cristina",
        "Cruz",
        "Custodia",
        "Cándida",
        "Dafne",
        "Dalila",
        "Daniela",
        "Delfina",
        "Delia",
        "Diana",
        "Dionisia",
        "Dolores",
        "Dominga",
        "Domitila",
        "Dora",
        "Dorita",
        "Dorotea",
        "Dulce",
        "Débora",
        "Edelmira",
        "Elba",
        "Elena",
        "Eli",
        "Eliana",
        "Eligia",
        "Elisa",
        "Elisabet",
        "Elodia",
        "Eloísa",
        "Elvira",
        "Ema",
        "Emelina",
        "Emilia",
        "Emiliana",
        "Emma",
        "Emperatriz",
        "Encarna",
        "Encarnacion",
        "Encarnación",
        "Encarnita",
        "Esmeralda",
        "Esperanza",
        "Estefanía",
        "Estela",
        "Ester",
        "Esther",
        "Estrella",
        "Etelvina",
        "Eufemia",
        "Eugenia",
        "Eulalia",
        "Eusebia",
        "Eva",
        "Eva María",
        "Evangelina",
        "Evelia",
        "Evita",
        "Fabiana",
        "Fabiola",
        "Fanny",
        "Febe",
        "Felicia",
        "Feliciana",
        "Felicidad",
        "Felipa",
        "Felisa",
        "Fernanda",
        "Fidela",
        "Filomena",
        "Flavia",
        "Flor",
        "Flora",
        "Florencia",
        "Florentina",
        "Florina",
        "Florinda",
        "Fortunata",
        "Francisca",
        "Fátima",
        "Gabriela",
        "Gala",
        "Gema",
        "Genoveva",
        "Georgina",
        "Gertrudis",
        "Gisela",
        "Gloria",
        "Gracia",
        "Graciana",
        "Graciela",
        "Griselda",
        "Guadalupe",
        "Guiomar",
        "Haydée",
        "Herminia",
        "Hilda",
        "Hortensia",
        "Ignacia",
        "Ileana",
        "Imelda",
        "Inmaculada",
        "Inés",
        "Irene",
        "Iris",
        "Irma",
        "Isa",
        "Isabel",
        "Isabela",
        "Isaura",
        "Isidora",
        "Itziar",
        "Jacinta",
        "Javiera",
        "Jennifer",
        "Jenny",
        "Jessica",
        "Jesusa",
        "Jimena",
        "Joaquina",
        "Jordana",
        "Josefa",
        "Josefina",
        "José",
        "Jovita",
        "Juana",
        "Juanita",
        "Judith",
        "Julia",
        "Juliana",
        "Julie",
        "Julieta",
        "Lara",
        "Laura",
        "Leandra",
        "Leire",
        "Leocadia",
        "Leonor",
        "Leticia",
        "Leyre",
        "Lidia",
        "Ligia",
        "Lilia",
        "Liliana",
        "Lina",
        "Loida",
        "Lola",
        "Lorena",
        "Lorenza",
        "Loreto",
        "Lourdes",
        "Luciana",
        "Lucila",
        "Lucía",
        "Luisa",
        "Luisina",
        "Luna",
        "Lupe",
        "Lupita",
        "Luz",
        "Macarena",
        "Macaria",
        "Magdalena",
        "Maite",
        "Malena",
        "Mamen",
        "Manola",
        "Manu",
        "Manuela",
        "Manuelita",
        "Mar",
        "Marcela",
        "Marcia",
        "Margarita",
        "Mariana",
        "Marianela",
        "Maribel",
        "Maricela",
        "Maricruz",
        "Marina",
        "Marisa",
        "Marisela",
        "Marisol",
        "Maristela",
        "Marita",
        "Marta",
        "Martina",
        "Martirio",
        "María",
        "María Belén",
        "María Carmen",
        "María Cristina",
        "María Del Carmen",
        "María Dolores",
        "María Fernanda",
        "María Jesús",
        "María José",
        "María Luisa",
        "María Manuela",
        "María Pilar",
        "María Teresa",
        "María Ángeles",
        "Matilde",
        "Maura",
        "Maxi",
        "Mayte",
        "Melania",
        "Melisa",
        "Mercedes",
        "Merche",
        "Micaela",
        "Miguela",
        "Milagros",
        "Mireia",
        "Miriam",
        "Mirta",
        "Modesta",
        "Montserrat",
        "Morena",
        "Máxima",
        "Mónica",
        "Nadia",
        "Narcisa",
        "Natalia",
        "Natividad",
        "Nayara",
        "Nazaret",
        "Nerea",
        "Nereida",
        "Nicolasa",
        "Nidia",
        "Nieves",
        "Nilda",
        "Noa",
        "Noelia",
        "Noemí",
        "Nuria",
        "Nydia",
        "Nélida",
        "Obdulia",
        "Octavia",
        "Odalis",
        "Odalys",
        "Ofelia",
        "Olalla",
        "Olga",
        "Olimpia",
        "Olivia",
        "Oriana",
        "Otilia",
        "Paca",
        "Pacífica",
        "Palmira",
        "Paloma",
        "Paola",
        "Pascuala",
        "Pastora",
        "Patricia",
        "Paula",
        "Paulina",
        "Paz",
        "Pepita",
        "Perla",
        "Perlita",
        "Petrona",
        "Piedad",
        "Pilar",
        "Pili",
        "Primitiva",
        "Priscila",
        "Prudencia",
        "Purificación",
        "Pía",
        "Rafaela",
        "Ramona",
        "Raquel",
        "Rebeca",
        "Regina",
        "Reina",
        "Remedios",
        "Renata",
        "Reyes",
        "Reyna",
        "Ricarda",
        "Rita",
        "Roberta",
        "Rocío",
        "Rosa",
        "Rosa María",
        "Rosalina",
        "Rosalinda",
        "Rosalva",
        "Rosalía",
        "Rosario",
        "Rosaura",
        "Rosenda",
        "Roxana",
        "Rufina",
        "Ruperta",
        "Ruth",
        "Sabina",
        "Salomé",
        "Salud",
        "Samanta",
        "Sandra",
        "Sara",
        "Sarita",
        "Saturnina",
        "Selena",
        "Serafina",
        "Silvia",
        "Socorro",
        "Sofía",
        "Sol",
        "Soledad",
        "Sonia",
        "Soraya",
        "Susana",
        "Susanita",
        "Tamara",
        "Tania",
        "Tatiana",
        "Tecla",
        "Teodora",
        "Tere",
        "Teresa",
        "Teresita",
        "Teófila",
        "Tomasa",
        "Trini",
        "Trinidad",
        "Valentina",
        "Valeria",
        "Vanesa",
        "Vera",
        "Verónica",
        "Vicenta",
        "Victoria",
        "Vilma",
        "Violeta",
        "Virginia",
        "Visitación",
        "Viviana",
        "Ximena",
        "Xiomara",
        "Yaiza",
        "Yolanda",
        "Yésica",
        "Yéssica",
        "Zaida",
        "Zaira",
        "Zoraida",
        "África",
        "Ágata",
        "Águeda",
        "Ámbar",
        "Ángela",
        "Ángeles",
        "Áurea",
        "Íngrid",
        "Úrsula",
    )

    first_names = first_names_male + first_names_female

    last_names = (
        "Abad",
        "Abascal",
        "Abella",
        "Abellán",
        "Abril",
        "Acedo",
        "Acero",
        "Acevedo",
        "Acosta",
        "Acuña",
        "Adadia",
        "Adán",
        "Aguado",
        "Agudo",
        "Aguilar",
        "Aguilera",
        "Aguiló",
        "Aguirre",
        "Agullo",
        "Agustí",
        "Agustín",
        "Alarcón",
        "Alba",
        "Alberdi",
        "Albero",
        "Alberola",
        "Alberto",
        "Alcalde",
        "Alcalá",
        "Alcaraz",
        "Alcolea",
        "Alcántara",
        "Alcázar",
        "Alegre",
        "Alegria",
        "Alemany",
        "Alemán",
        "Alfaro",
        "Alfonso",
        "Aliaga",
        "Aller",
        "Almagro",
        "Almansa",
        "Almazán",
        "Almeida",
        "Alonso",
        "Alsina",
        "Alvarado",
        "Alvarez",
        "Amador",
        "Amat",
        "Amaya",
        "Amigó",
        "Amo",
        "Amor",
        "Amores",
        "Amorós",
        "Anaya",
        "Andrade",
        "Andres",
        "Andreu",
        "Andrés",
        "Anglada",
        "Anguita",
        "Angulo",
        "Antón",
        "Antúnez",
        "Aparicio",
        "Aragonés",
        "Aragón",
        "Aramburu",
        "Arana",
        "Aranda",
        "Araujo",
        "Arce",
        "Arco",
        "Arcos",
        "Arellano",
        "Arenas",
        "Arias",
        "Ariza",
        "Ariño",
        "Arjona",
        "Armas",
        "Armengol",
        "Arnaiz",
        "Arnal",
        "Arnau",
        "Aroca",
        "Arranz",
        "Arregui",
        "Arribas",
        "Arrieta",
        "Arroyo",
        "Arteaga",
        "Artigas",
        "Arévalo",
        "Asenjo",
        "Asensio",
        "Atienza",
        "Avilés",
        "Ayala",
        "Ayllón",
        "Ayuso",
        "Azcona",
        "Aznar",
        "Azorin",
        "Badía",
        "Baena",
        "Baeza",
        "Balaguer",
        "Ballester",
        "Ballesteros",
        "Baquero",
        "Barba",
        "Barbero",
        "Barberá",
        "Barceló",
        "Barco",
        "Barragán",
        "Barral",
        "Barranco",
        "Barreda",
        "Barrena",
        "Barrera",
        "Barriga",
        "Barrio",
        "Barrios",
        "Barros",
        "Barroso",
        "Bartolomé",
        "Baró",
        "Barón",
        "Bas",
        "Bastida",
        "Batalla",
        "Batlle",
        "Bautista",
        "Bauzà",
        "Bayo",
        "Bayona",
        "Bayón",
        "Baños",
        "Becerra",
        "Bejarano",
        "Belda",
        "Bellido",
        "Bello",
        "Belmonte",
        "Beltran",
        "Beltrán",
        "Benavent",
        "Benavente",
        "Benavides",
        "Benet",
        "Benitez",
        "Benito",
        "Benítez",
        "Berenguer",
        "Bermejo",
        "Bermudez",
        "Bermúdez",
        "Bernad",
        "Bernal",
        "Bernat",
        "Berrocal",
        "Bertrán",
        "Bilbao",
        "Blanca",
        "Blanch",
        "Blanco",
        "Blanes",
        "Blasco",
        "Blazquez",
        "Blázquez",
        "Boada",
        "Boix",
        "Bolaños",
        "Bonet",
        "Bonilla",
        "Borja",
        "Borrego",
        "Borrell",
        "Borrás",
        "Bosch",
        "Botella",
        "Bou",
        "Bravo",
        "Briones",
        "Bru",
        "Buendía",
        "Bueno",
        "Burgos",
        "Busquets",
        "Bustamante",
        "Bustos",
        "Báez",
        "Bárcena",
        "Caballero",
        "Cabanillas",
        "Cabañas",
        "Cabello",
        "Cabeza",
        "Cabezas",
        "Cabo",
        "Cabrera",
        "Cabrero",
        "Cadenas",
        "Cal",
        "Calatayud",
        "Calderon",
        "Calderón",
        "Calleja",
        "Calvet",
        "Calvo",
        "Calzada",
        "Camacho",
        "Camino",
        "Campillo",
        "Campo",
        "Campos",
        "Campoy",
        "Camps",
        "Canales",
        "Canals",
        "Canet",
        "Cano",
        "Cantero",
        "Cantón",
        "Caparrós",
        "Capdevila",
        "Carbajo",
        "Carballo",
        "Carbonell",
        "Carbó",
        "Cardona",
        "Carlos",
        "Carmona",
        "Carnero",
        "Caro",
        "Carpio",
        "Carranza",
        "Carrasco",
        "Carrera",
        "Carreras",
        "Carretero",
        "Carreño",
        "Carrillo",
        "Carrión",
        "Carro",
        "Carvajal",
        "Casado",
        "Casal",
        "Casals",
        "Casanova",
        "Casanovas",
        "Casares",
        "Casas",
        "Cases",
        "Castañeda",
        "Castejón",
        "Castell",
        "Castellanos",
        "Castells",
        "Castelló",
        "Castilla",
        "Castillo",
        "Castrillo",
        "Castro",
        "Catalá",
        "Catalán",
        "Cazorla",
        "Cañas",
        "Cañellas",
        "Cañete",
        "Cañizares",
        "Cepeda",
        "Cerdá",
        "Cerdán",
        "Cerezo",
        "Cerro",
        "Cervantes",
        "Cervera",
        "Chacón",
        "Chamorro",
        "Chaparro",
        "Chaves",
        "Checa",
        "Chico",
        "Cid",
        "Cifuentes",
        "Cisneros",
        "Clavero",
        "Clemente",
        "Cobo",
        "Cobos",
        "Coca",
        "Codina",
        "Coello",
        "Coll",
        "Collado",
        "Colom",
        "Coloma",
        "Colomer",
        "Comas",
        "Company",
        "Conde",
        "Conesa",
        "Contreras",
        "Corbacho",
        "Cordero",
        "Cornejo",
        "Corominas",
        "Coronado",
        "Corral",
        "Correa",
        "Cortes",
        "Cortina",
        "Cortés",
        "Costa",
        "Crespi",
        "Crespo",
        "Criado",
        "Cruz",
        "Cuadrado",
        "Cuenca",
        "Cuervo",
        "Cuesta",
        "Cueto",
        "Cuevas",
        "Cuéllar",
        "Cáceres",
        "Cámara",
        "Cánovas",
        "Cárdenas",
        "Céspedes",
        "Córdoba",
        "Cózar",
        "Dalmau",
        "Daza",
        "Delgado",
        "Diaz",
        "Diego",
        "Diez",
        "Diéguez",
        "Domingo",
        "Dominguez",
        "Doménech",
        "Domínguez",
        "Donaire",
        "Donoso",
        "Duarte",
        "Dueñas",
        "Duque",
        "Duran",
        "Durán",
        "Dávila",
        "Díaz",
        "Díez",
        "Echevarría",
        "Echeverría",
        "Egea",
        "Elorza",
        "Elías",
        "Enríquez",
        "Escalona",
        "Escamilla",
        "Escobar",
        "Escolano",
        "Escribano",
        "Escrivá",
        "Escudero",
        "Espada",
        "Esparza",
        "España",
        "Español",
        "Espejo",
        "Espinosa",
        "Esteban",
        "Esteve",
        "Estevez",
        "Estrada",
        "Estévez",
        "Exposito",
        "Expósito",
        "Fabra",
        "Fabregat",
        "Fajardo",
        "Falcó",
        "Falcón",
        "Farré",
        "Feijoo",
        "Feliu",
        "Fernandez",
        "Fernández",
        "Ferrando",
        "Ferrer",
        "Ferrera",
        "Ferreras",
        "Ferrero",
        "Ferrán",
        "Ferrández",
        "Ferrándiz",
        "Figueras",
        "Figueroa",
        "Figuerola",
        "Fiol",
        "Flor",
        "Flores",
        "Folch",
        "Fonseca",
        "Font",
        "Fortuny",
        "Franch",
        "Francisco",
        "Franco",
        "Frutos",
        "Frías",
        "Fuente",
        "Fuentes",
        "Fuertes",
        "Fuster",
        "Fábregas",
        "Gabaldón",
        "Galan",
        "Galiano",
        "Galindo",
        "Gallardo",
        "Gallart",
        "Gallego",
        "Gallo",
        "Galvez",
        "Galván",
        "Galán",
        "Garay",
        "Garcia",
        "Garcés",
        "García",
        "Gargallo",
        "Garmendia",
        "Garrido",
        "Garriga",
        "Garzón",
        "Gascón",
        "Gaya",
        "Gelabert",
        "Gibert",
        "Gil",
        "Gilabert",
        "Gimenez",
        "Gimeno",
        "Giménez",
        "Giner",
        "Giralt",
        "Girona",
        "Girón",
        "Gisbert",
        "Godoy",
        "Goicoechea",
        "Gomez",
        "Gomila",
        "Gomis",
        "Gonzalez",
        "Gonzalo",
        "González",
        "Gordillo",
        "Goñi",
        "Gracia",
        "Granados",
        "Grande",
        "Gras",
        "Grau",
        "Gual",
        "Guardia",
        "Guardiola",
        "Guerra",
        "Guerrero",
        "Guijarro",
        "Guillen",
        "Guillén",
        "Guitart",
        "Gutierrez",
        "Gutiérrez",
        "Guzman",
        "Guzmán",
        "Gálvez",
        "Gámez",
        "Gárate",
        "Gómez",
        "Haro",
        "Heras",
        "Heredia",
        "Hernandez",
        "Hernando",
        "Hernández",
        "Herranz",
        "Herrera",
        "Herrero",
        "Hervia",
        "Hervás",
        "Hidalgo",
        "Hierro",
        "Higueras",
        "Hoyos",
        "Hoz",
        "Huerta",
        "Huertas",
        "Huguet",
        "Hurtado",
        "Ibarra",
        "Ibañez",
        "Iborra",
        "Ibáñez",
        "Iglesia",
        "Iglesias",
        "Infante",
        "Iniesta",
        "Iriarte",
        "Isern",
        "Izaguirre",
        "Izquierdo",
        "Iñiguez",
        "Jara",
        "Jaume",
        "Jaén",
        "Jerez",
        "Jimenez",
        "Jiménez",
        "Jordá",
        "Jordán",
        "Jove",
        "Jover",
        "Juan",
        "Juliá",
        "Julián",
        "Jurado",
        "Juárez",
        "Jáuregui",
        "Jódar",
        "Lago",
        "Laguna",
        "Lamas",
        "Landa",
        "Lara",
        "Larrañaga",
        "Larrea",
        "Lasa",
        "Lastra",
        "Leal",
        "Ledesma",
        "Leiva",
        "Leon",
        "Lerma",
        "León",
        "Lillo",
        "Linares",
        "Llabrés",
        "Lladó",
        "Llamas",
        "Llano",
        "Llanos",
        "Lledó",
        "Llobet",
        "Llopis",
        "Llorens",
        "Llorente",
        "Lloret",
        "Lluch",
        "Lobato",
        "Lobo",
        "Lopez",
        "Lorenzo",
        "Losa",
        "Losada",
        "Lozano",
        "Lucas",
        "Lucena",
        "Luján",
        "Lumbreras",
        "Luna",
        "Luque",
        "Luz",
        "Luís",
        "López",
        "Machado",
        "Macias",
        "Macías",
        "Madrid",
        "Madrigal",
        "Maestre",
        "Maldonado",
        "Malo",
        "Mancebo",
        "Manjón",
        "Manrique",
        "Manso",
        "Manuel",
        "Manzanares",
        "Manzano",
        "Marco",
        "Marcos",
        "Marin",
        "Mariscal",
        "Mariño",
        "Marquez",
        "Marqués",
        "Marti",
        "Martin",
        "Martinez",
        "Martorell",
        "Martí",
        "Martín",
        "Martínez",
        "Marí",
        "Marín",
        "Mas",
        "Mascaró",
        "Mata",
        "Matas",
        "Mate",
        "Mateo",
        "Mateos",
        "Mateu",
        "Mayo",
        "Mayol",
        "Mayoral",
        "Maza",
        "Medina",
        "Melero",
        "Meléndez",
        "Mena",
        "Mendez",
        "Mendizábal",
        "Mendoza",
        "Menendez",
        "Menéndez",
        "Mercader",
        "Merino",
        "Mesa",
        "Miguel",
        "Milla",
        "Millán",
        "Mir",
        "Miralles",
        "Miranda",
        "Miró",
        "Moles",
        "Molina",
        "Moliner",
        "Molins",
        "Moll",
        "Monreal",
        "Montalbán",
        "Montaña",
        "Montenegro",
        "Montero",
        "Montes",
        "Montesinos",
        "Montoya",
        "Montserrat",
        "Mora",
        "Moraleda",
        "Morales",
        "Morante",
        "Morata",
        "Morcillo",
        "Morell",
        "Moreno",
        "Morera",
        "Morillo",
        "Morán",
        "Mosquera",
        "Moya",
        "Mulet",
        "Mur",
        "Murcia",
        "Murillo",
        "Muro",
        "Muñoz",
        "Mármol",
        "Márquez",
        "Méndez",
        "Mínguez",
        "Múgica",
        "Múñiz",
        "Nadal",
        "Naranjo",
        "Narváez",
        "Navarrete",
        "Navarro",
        "Navas",
        "Nebot",
        "Neira",
        "Nevado",
        "Nicolau",
        "Nicolás",
        "Nieto",
        "Niño",
        "Nogueira",
        "Noguera",
        "Nogués",
        "Noriega",
        "Novoa",
        "Nuñez",
        "Núñez",
        "Ocaña",
        "Ochoa",
        "Ojeda",
        "Oliva",
        "Olivares",
        "Oliver",
        "Olivera",
        "Oliveras",
        "Olivé",
        "Oller",
        "Olmedo",
        "Olmo",
        "Ordóñez",
        "Orozco",
        "Ortega",
        "Ortiz",
        "Ortuño",
        "Osorio",
        "Osuna",
        "Otero",
        "Pablo",
        "Pacheco",
        "Padilla",
        "Pagès",
        "Palacio",
        "Palacios",
        "Palau",
        "Pallarès",
        "Palma",
        "Palmer",
        "Palomar",
        "Palomares",
        "Palomino",
        "Palomo",
        "Paniagua",
        "Pardo",
        "Paredes",
        "Pareja",
        "Parejo",
        "Parra",
        "Pascual",
        "Pastor",
        "Patiño",
        "Pavón",
        "Paz",
        "Pazos",
        "Pedraza",
        "Pedrero",
        "Pedro",
        "Pedrosa",
        "Peinado",
        "Peiró",
        "Pelayo",
        "Pellicer",
        "Peláez",
        "Pera",
        "Peral",
        "Perales",
        "Peralta",
        "Perea",
        "Pereira",
        "Perelló",
        "Perera",
        "Perez",
        "Peña",
        "Peñalver",
        "Peñas",
        "Pi",
        "Pina",
        "Pineda",
        "Pinedo",
        "Pinilla",
        "Pino",
        "Pinto",
        "Pintor",
        "Piquer",
        "Pizarro",
        "Piña",
        "Piñeiro",
        "Piñol",
        "Pla",
        "Plana",
        "Planas",
        "Plaza",
        "Pol",
        "Polo",
        "Pomares",
        "Pombo",
        "Ponce",
        "Pons",
        "Pont",
        "Porcel",
        "Porras",
        "Porta",
        "Portero",
        "Portillo",
        "Posada",
        "Pou",
        "Poza",
        "Pozo",
        "Pozuelo",
        "Prada",
        "Prado",
        "Prat",
        "Prats",
        "Priego",
        "Prieto",
        "Puente",
        "Puerta",
        "Puga",
        "Puig",
        "Pujadas",
        "Pujol",
        "Pulido",
        "Páez",
        "Pérez",
        "Quero",
        "Querol",
        "Quesada",
        "Quevedo",
        "Quintana",
        "Quintanilla",
        "Quintero",
        "Quiroga",
        "Quirós",
        "Ramirez",
        "Ramis",
        "Ramos",
        "Ramírez",
        "Ramón",
        "Raya",
        "Real",
        "Rebollo",
        "Recio",
        "Redondo",
        "Reguera",
        "Reig",
        "Reina",
        "Requena",
        "Revilla",
        "Rey",
        "Reyes",
        "Riba",
        "Ribas",
        "Ribera",
        "Ribes",
        "Ricart",
        "Rico",
        "Riera",
        "Rincón",
        "Rios",
        "Ripoll",
        "Riquelme",
        "Rius",
        "Rivas",
        "Rivera",
        "Rivero",
        "Robledo",
        "Robles",
        "Roca",
        "Rocamora",
        "Rocha",
        "Roda",
        "Rodrigo",
        "Rodriguez",
        "Rodríguez",
        "Roig",
        "Rojas",
        "Roldan",
        "Roldán",
        "Roma",
        "Roman",
        "Romero",
        "Romeu",
        "Román",
        "Ropero",
        "Ros",
        "Rosa",
        "Rosado",
        "Rosales",
        "Rosell",
        "Roselló",
        "Rosselló",
        "Roura",
        "Rovira",
        "Royo",
        "Rozas",
        "Ruano",
        "Rubio",
        "Rueda",
        "Ruiz",
        "Río",
        "Ríos",
        "Ródenas",
        "Saavedra",
        "Sabater",
        "Sacristán",
        "Saez",
        "Sainz",
        "Sala",
        "Salamanca",
        "Salas",
        "Salazar",
        "Salcedo",
        "Saldaña",
        "Sales",
        "Salgado",
        "Salinas",
        "Salmerón",
        "Salom",
        "Salvador",
        "Salvà",
        "Samper",
        "Sanabria",
        "Sanchez",
        "Sancho",
        "Sandoval",
        "Sanjuan",
        "Sanmartín",
        "Sanmiguel",
        "Sans",
        "Santamaria",
        "Santamaría",
        "Santana",
        "Santiago",
        "Santos",
        "Sanz",
        "Sarabia",
        "Sarmiento",
        "Sastre",
        "Saura",
        "Sebastián",
        "Seco",
        "Sedano",
        "Segarra",
        "Segovia",
        "Segura",
        "Seguí",
        "Serna",
        "Serra",
        "Serrano",
        "Sevilla",
        "Sevillano",
        "Sierra",
        "Silva",
        "Simó",
        "Sobrino",
        "Sola",
        "Solana",
        "Solano",
        "Soler",
        "Solera",
        "Solsona",
        "Solé",
        "Solís",
        "Somoza",
        "Soria",
        "Soriano",
        "Sosa",
        "Sotelo",
        "Soto",
        "Suarez",
        "Sureda",
        "Suárez",
        "Sáenz",
        "Sáez",
        "Sánchez",
        "Taboada",
        "Talavera",
        "Tamarit",
        "Tamayo",
        "Tapia",
        "Tejada",
        "Tejedor",
        "Tejera",
        "Tejero",
        "Tello",
        "Tena",
        "Tenorio",
        "Terrón",
        "Teruel",
        "Tirado",
        "Toledo",
        "Tolosa",
        "Tomas",
        "Tomás",
        "Tomé",
        "Tormo",
        "Toro",
        "Torralba",
        "Torre",
        "Torrecilla",
        "Torrens",
        "Torrent",
        "Torrents",
        "Torres",
        "Torrijos",
        "Tovar",
        "Trillo",
        "Trujillo",
        "Tudela",
        "Tur",
        "Téllez",
        "Ugarte",
        "Ureña",
        "Uriarte",
        "Uribe",
        "Urrutia",
        "Uría",
        "Valbuena",
        "Valcárcel",
        "Valderrama",
        "Valdés",
        "Valencia",
        "Valenciano",
        "Valentín",
        "Valenzuela",
        "Valera",
        "Valero",
        "Vall",
        "Valle",
        "Vallejo",
        "Valls",
        "Vallés",
        "Valverde",
        "Vaquero",
        "Vara",
        "Varela",
        "Vargas",
        "Vazquez",
        "Vega",
        "Velasco",
        "Velázquez",
        "Vendrell",
        "Vera",
        "Verdejo",
        "Verdugo",
        "Verdú",
        "Vergara",
        "Viana",
        "Vicens",
        "Vicente",
        "Vidal",
        "Vigil",
        "Vila",
        "Vilalta",
        "Vilanova",
        "Vilaplana",
        "Vilar",
        "Villa",
        "Villalba",
        "Villalobos",
        "Villalonga",
        "Villanueva",
        "Villar",
        "Villaverde",
        "Villegas",
        "Villena",
        "Vives",
        "Vizcaíno",
        "Viña",
        "Viñas",
        "Vázquez",
        "Vélez",
        "Yuste",
        "Yáñez",
        "Zabala",
        "Zabaleta",
        "Zamora",
        "Zamorano",
        "Zapata",
        "Zaragoza",
        "Zorrilla",
        "Zurita",
        "Águila",
        "Álamo",
        "Álvarez",
        "Álvaro",
        "Ángel",
        "Ávila",
    )

    prefixes = ("de", "del")



from .. import Provider as DateTimeProvider


class Provider(DateTimeProvider):
    def day_of_week(self) -> str:
        day = self.date("%w")
        DAY_NAMES = {
            "0": "Nedelja",
            "1": "Ponedeljek",
            "2": "Torek",
            "3": "Sreda",
            "4": "Četrtek",
            "5": "Petek",
            "6": "Sobota",
        }
        return DAY_NAMES[day]

    def month_name(self) -> str:
        month = self.month()
        MONTH_NAMES = {
            "01": "Januar",
            "02": "Februar",
            "03": "Marec",
            "04": "April",
            "05": "Maj",
            "06": "Junij",
            "07": "Julij",
            "08": "Avgust",
            "09": "September",
            "10": "Oktober",
            "11": "November",
            "12": "December",
        }
        return MONTH_NAMES[month]



import warnings

from datetime import datetime
from typing import Optional

from ....typing import DateParseType
from .. import Provider as DateParseTypeProvider

# thai_strftime() code adapted from
# https://gist.github.com/bact/b8afe49cb1ae62913e6c1e899dcddbdb
# (Same code base with PyThaiNLP 2.x)
# Public Domain or CC0 1.0 Universal

_TH_ABBR_WEEKDAYS = ["จ", "อ", "พ", "พฤ", "ศ", "ส", "อา"]
_TH_FULL_WEEKDAYS = [
    "วันจันทร์",
    "วันอังคาร",
    "วันพุธ",
    "วันพฤหัสบดี",
    "วันศุกร์",
    "วันเสาร์",
    "วันอาทิตย์",
]

_TH_ABBR_MONTHS = [
    "ม.ค.",
    "ก.พ.",
    "มี.ค.",
    "เม.ย.",
    "พ.ค.",
    "มิ.ย.",
    "ก.ค.",
    "ส.ค.",
    "ก.ย.",
    "ต.ค.",
    "พ.ย.",
    "ธ.ค.",
]
_TH_FULL_MONTHS = [
    "มกราคม",
    "กุมภาพันธ์",
    "มีนาคม",
    "เมษายน",
    "พฤษภาคม",
    "มิถุนายน",
    "กรกฎาคม",
    "สิงหาคม",
    "กันยายน",
    "ตุลาคม",
    "พฤศจิกายน",
    "ธันวาคม",
]

_HA_TH_DIGITS = str.maketrans("0123456789", "๐๑๒๓๔๕๖๗๘๙")
_BE_AD_DIFFERENCE = 543

_NEED_L10N = "AaBbCcDFGgvXxYy+"  # flags that need localization
_EXTENSIONS = "EO-_0^#"  # extension flags


# Standard conversion support for thai_strftime()
def _std_strftime(dt_obj: datetime, fmt_char: str) -> str:
    """
    Standard datetime.strftime() with normalization and exception handling.
    """
    str_ = ""
    try:
        str_ = dt_obj.strftime(f"%{fmt_char}")
        if not str_ or str_ == f"%{fmt_char}":
            # normalize outputs for unsupported directives
            # in different platforms
            # "%Q" may result "%Q", "Q", or "", make it "Q"
            str_ = fmt_char
    except ValueError as err:  # pragma: no cover
        # Unsupported directives may raise ValueError on Windows,
        # in that case just use the fmt_char
        warnings.warn(
            (f"String format directive unknown/not support: %{fmt_char}" f"The system raises this ValueError: {err}"),
            UserWarning,
        )
        str_ = fmt_char
    return str_


# Thai conversion support for thai_strftime()
def _thai_strftime(
    dt_obj: datetime,
    fmt_char: str,
    buddhist_era: bool = True,
) -> str:
    """
    Conversion support for thai_strftime().

    The fmt_char should be in _NEED_L10N when call this function.
    """
    str_ = ""
    year = dt_obj.year
    if buddhist_era:
        year = year + _BE_AD_DIFFERENCE

    if fmt_char == "A":
        # National representation of the full weekday name
        str_ = _TH_FULL_WEEKDAYS[dt_obj.weekday()]
    elif fmt_char == "a":
        # National representation of the abbreviated weekday
        str_ = _TH_ABBR_WEEKDAYS[dt_obj.weekday()]
    elif fmt_char == "B":
        # National representation of the full month name
        str_ = _TH_FULL_MONTHS[dt_obj.month - 1]
    elif fmt_char == "b":
        # National representation of the abbreviated month name
        str_ = _TH_ABBR_MONTHS[dt_obj.month - 1]
    elif fmt_char == "C":
        # Thai Buddhist century (AD+543)/100 + 1 as decimal number;
        str_ = str(int(year / 100) + 1).zfill(2)
    elif fmt_char == "c":
        # Locale’s appropriate date and time representation
        # Wed  6 Oct 01:40:00 1976
        # พ   6 ต.ค. 01:40:00 2519  <-- left-aligned weekday, right-aligned day
        str_ = (
            f"{_TH_ABBR_WEEKDAYS[dt_obj.weekday()]:<2} {dt_obj.day:>2} "
            f"{_TH_ABBR_MONTHS[dt_obj.month - 1]} {dt_obj:%H:%M:%S} {year:04}"
        )
    elif fmt_char == "D":
        # Equivalent to ``%m/%d/%y''
        str_ = f"{dt_obj:%m/%d}/{year % 100:02}"
    elif fmt_char == "F":
        # Equivalent to ``%Y-%m-%d''
        str_ = f"{year:04}-{dt_obj:%m-%d}"
    elif fmt_char == "G":
        # ISO 8601 year with century representing the year that contains
        # the greater part of the ISO week (%V). Monday as the first day
        # of the week.
        year_G = int(dt_obj.strftime("%G"))
        if buddhist_era:
            year_G = year_G + _BE_AD_DIFFERENCE
        str_ = f"{year_G:04}"
    elif fmt_char == "g":
        # Same year as in ``%G'',
        # but as a decimal number without century (00-99).
        year_G = int(dt_obj.strftime("%G"))
        if buddhist_era:
            year_G = year_G + _BE_AD_DIFFERENCE
        str_ = f"{year_G % 100:02}"
    elif fmt_char == "v":
        # BSD extension, ' 6-Oct-1976'
        str_ = f"{dt_obj.day:>2}-{_TH_ABBR_MONTHS[dt_obj.month - 1]}-{year:04}"
    elif fmt_char == "X":
        # Locale’s appropriate time representation.
        str_ = f"{dt_obj:%H:%M:%S}"
    elif fmt_char == "x":
        # Locale’s appropriate date representation.
        str_ = f"{dt_obj:%d/%m}/{year:04}"
    elif fmt_char == "Y":
        # Year with century
        str_ = f"{year:04}"
    elif fmt_char == "y":
        # Year without century
        str_ = f"{year % 100:02}"
    elif fmt_char == "+":
        # National representation of the date and time
        # (the format is similar to that produced by date(1))
        # Wed  6 Oct 1976 01:40:00
        str_ = (
            f"{_TH_ABBR_WEEKDAYS[dt_obj.weekday()]:<2} {dt_obj.day:>2} "
            f"{_TH_ABBR_MONTHS[dt_obj.month - 1]} {year} {dt_obj:%H:%M:%S}"
        )

    return str_


def thai_strftime(
    dt_obj: datetime,
    fmt: str = "%-d %b %Y",
    thai_digit: bool = False,
    buddhist_era: bool = True,
) -> str:
    """
    Convert :class:`datetime.datetime` into Thai date and time format.

    The formatting directives are similar to :func:`datatime.strrftime`.

    This function uses Thai names and Thai Buddhist Era for these directives:
        * **%a** - abbreviated weekday name
        (i.e. "จ", "อ", "พ", "พฤ", "ศ", "ส", "อา")
        * **%A** - full weekday name
        (i.e. "วันจันทร์", "วันอังคาร", "วันเสาร์", "วันอาทิตย์")
        * **%b** - abbreviated month name
        (i.e. "ม.ค.", "ก.พ.", "มี.ค.", "เม.ย.", "พ.ค.", "มิ.ย.", "ธ.ค.")
        * **%B** - full month name
        (i.e. "มกราคม", "กุมภาพันธ์", "พฤศจิกายน", "ธันวาคม",)
        * **%y** - year without century (i.e. "56", "10")
        * **%Y** - year with century (i.e. "2556", "2410")
        * **%c** - date and time representation
        (i.e. "พ   6 ต.ค. 01:40:00 2519")
        * **%v** - short date representation
        (i.e. " 6-ม.ค.-2562", "27-ก.พ.-2555")
    """
    thaidate_parts = []

    i = 0
    fmt_len = len(fmt)
    while i < fmt_len:
        str_ = ""
        if fmt[i] == "%":
            j = i + 1
            if j < fmt_len:
                fmt_char = fmt[j]
                if fmt_char in _NEED_L10N:  # requires localization?
                    str_ = _thai_strftime(dt_obj, fmt_char, buddhist_era)
                elif fmt_char in _EXTENSIONS:
                    fmt_char_ext = fmt_char
                    k = j + 1
                    if k < fmt_len:
                        fmt_char = fmt[k]
                        if fmt_char in _NEED_L10N:
                            str_ = _thai_strftime(
                                dt_obj,
                                fmt_char,
                                buddhist_era,
                            )
                        else:
                            str_ = _std_strftime(dt_obj, fmt_char)

                        if fmt_char_ext == "-":
                            # GNU libc extension,
                            # no padding
                            if str_[0] and str_[0] in " 0":
                                str_ = str_[1:]
                        elif fmt_char_ext == "_":
                            # GNU libc extension,
                            # explicitly specify space (" ") for padding
                            if str_[0] and str_[0] == "0":
                                str_ = " " + str_[1:]
                        elif fmt_char_ext == "0":
                            # GNU libc extension,
                            # explicitly specify zero ("0") for padding
                            if str_[0] and str_[0] == " ":
                                str_ = "0" + str_[1:]
                        elif fmt_char_ext == "^":
                            # GNU libc extension,
                            # convert to upper case
                            str_ = str_.upper()
                        elif fmt_char_ext == "#":
                            # GNU libc extension,
                            # swap case - useful for %Z
                            str_ = str_.swapcase()
                        elif fmt_char_ext == "E":
                            # POSIX extension,
                            # uses the locale's alternative representation
                            # Not implemented yet
                            pass
                        elif fmt_char_ext == "O":
                            # POSIX extension,
                            # uses the locale's alternative numeric symbols
                            str_ = str_.translate(_HA_TH_DIGITS)
                        i = i + 1  # consume char after format char
                    else:
                        # format char at string's end has no meaning
                        str_ = fmt_char_ext
                else:  # not in _NEED_L10N nor _EXTENSIONS
                    # no known localization available, use Python's default
                    str_ = _std_strftime(dt_obj, fmt_char)

                i = i + 1  # consume char after "%"
            else:
                # % char at string's end has no meaning
                str_ = "%"
        else:
            str_ = fmt[i]

        thaidate_parts.append(str_)
        i = i + 1

    thaidate_text = "".join(thaidate_parts)

    if thai_digit:
        thaidate_text = thaidate_text.translate(_HA_TH_DIGITS)

    return thaidate_text


class Provider(DateParseTypeProvider):
    def date(
        self,
        pattern: str = "%-d %b %Y",
        end_datetime: Optional[DateParseType] = None,
        thai_digit: bool = False,
        buddhist_era: bool = True,
    ) -> str:
        """
        Get a date string between January 1, 1970 and now
        :param pattern format
        :param end_datetime datetime
        :param thai_digit use Thai digit or not (default: False)
        :param buddhist_era use Buddist era or not (default: True)
        :example '08 พ.ย. 2563'
        :example '๐๘ พ.ย. 2563' (thai_digit = True)
        :example '8 พฤศิจกายน 2020' (pattern: str = "%-d %B %Y", buddhist_era = False)
        """
        return thai_strftime(
            self.date_time(end_datetime=end_datetime),
            pattern,
            thai_digit,
            buddhist_era,
        )

    def time(
        self,
        pattern: str = "%H:%M:%S",
        end_datetime: Optional[DateParseType] = None,
        thai_digit: bool = False,
    ) -> str:
        """
        Get a time string (24h format by default)
        :param pattern format
        :param end_datetime datetime
        :param thai_digit use Thai digit or not (default: False)
        :example '15:02:34'
        :example '๑๕:๐๒:๓๔' (thai_digit = True)
        """
        return thai_strftime(
            self.date_time(end_datetime=end_datetime),
            pattern,
            thai_digit,
        )

    def century(self, thai_digit: bool = False, buddhist_era: bool = True) -> str:
        """
        :param thai_digit use Thai digit or not (default: False)
        :param buddhist_era use Buddist era or not (default: True)
        :example '20'
        """
        end_century = 22
        if buddhist_era:
            end_century = 26
        text = str(self.random_element(range(1, end_century)))
        if thai_digit:
            text = text.translate(_HA_TH_DIGITS)
        return text



class BaseFakerException(Exception):
    """The base exception for all Faker exceptions."""


class UniquenessException(BaseFakerException):
    """To avoid infinite loops, after a certain number of attempts,
    the "unique" attribute of the Proxy will throw this exception.
    """


class UnsupportedFeature(BaseFakerException):
    """The requested feature is not available on this system."""

    def __init__(self, msg: str, name: str) -> None:
        self.name = name
        super().__init__(msg)



from collections import OrderedDict
from operator import itemgetter
from typing import Tuple

from .. import Provider as PersonProvider


class Provider(PersonProvider):
    # link: http://dic.nicovideo.jp/a/日本人の名前一覧
    # link: http://www.meijiyasuda.co.jp/enjoy/ranking/
    first_name_female_pairs = (
        ("明美", "アケミ", "Akemi"),
        ("あすか", "アスカ", "Asuka"),
        ("香織", "カオリ", "Kaori"),
        ("加奈", "カナ", "Kana"),
        ("くみ子", "クミコ", "Kumiko"),
        ("さゆり", "サユリ", "Sayuri"),
        ("知実", "サトミ", "Satomi"),
        ("千代", "チヨ", "Chiyo"),
        ("直子", "ナオコ", "Naoko"),
        ("七夏", "ナナミ", "Nanami"),
        ("花子", "ハナコ", "Hanako"),
        ("春香", "ハルカ", "Haruka"),
        ("真綾", "マアヤ", "Maaya"),
        ("舞", "マイ", "Mai"),
        ("美加子", "ミカコ", "Mikako"),
        ("幹", "ミキ", "Miki"),
        ("桃子", "モモコ", "Momoko"),
        ("結衣", "ユイ", "Yui"),
        ("裕美子", "ユミコ", "Yumiko"),
        ("陽子", "ヨウコ", "Yoko"),
        ("里佳", "リカ", "Rika"),
    )

    # for backwards compatibility
    first_names_female = tuple(map(itemgetter(0), first_name_female_pairs))
    first_kana_names_female = tuple(map(itemgetter(1), first_name_female_pairs))
    first_romanized_names_female = tuple(map(itemgetter(2), first_name_female_pairs))

    first_name_male_pairs = (
        ("晃", "アキラ", "Akira"),
        ("篤司", "アツシ", "Atsushi"),
        ("治", "オサム", "Osamu"),
        ("和也", "カズヤ", "Kazuya"),
        ("京助", "キョウスケ", "Kyosuke"),
        ("健一", "ケンイチ", "Kenichi"),
        ("修平", "シュウヘイ", "Shohei"),
        ("翔太", "ショウタ", "Shota"),
        ("淳", "ジュン", "Jun"),
        ("聡太郎", "ソウタロウ", "Sotaro"),
        ("太一", "タイチ", "Taichi"),
        ("太郎", "タロウ", "Taro"),
        ("拓真", "タクマ", "Takuma"),
        ("翼", "ツバサ", "Tsubasa"),
        ("智也", "トモヤ", "Tomoya"),
        ("直樹", "ナオキ", "Naoki"),
        ("直人", "ナオト", "Naoto"),
        ("英樹", "ヒデキ", "Hideki"),
        ("浩", "ヒロシ", "Hiroshi"),
        ("学", "マナブ", "Manabu"),
        ("充", "ミツル", "Mituru"),
        ("稔", "ミノル", "Minoru"),
        ("裕樹", "ユウキ", "Yuki"),
        ("裕太", "ユウタ", "Yuta"),
        ("康弘", "ヤスヒロ", "Yasuhiro"),
        ("陽一", "ヨウイチ", "Yoichi"),
        ("洋介", "ヨウスケ", "Yosuke"),
        ("亮介", "リョウスケ", "Ryosuke"),
        ("涼平", "リョウヘイ", "Ryohei"),
        ("零", "レイ", "Rei"),
    )

    # for backwards compatibility
    first_names_male = tuple(map(itemgetter(0), first_name_male_pairs))
    first_kana_names_male = tuple(map(itemgetter(1), first_name_male_pairs))
    first_romanized_names_male = tuple(map(itemgetter(2), first_name_male_pairs))

    # for backwards compatibility
    first_names = first_names_male + first_names_female
    first_kana_names = first_kana_names_male + first_kana_names_female
    first_romanized_names = first_romanized_names_male + first_romanized_names_female

    first_name_pairs = first_name_male_pairs + first_name_female_pairs

    last_name_pairs = OrderedDict(
        (
            (("佐藤", "サトウ", "Sato"), 366803.0),
            (("鈴木", "スズキ", "Suzuki"), 321135),
            (("高橋", "タカハシ", "Takahashi"), 266782),
            (("田中", "タナカ", "Tanaka"), 245821),
            (("伊藤", "イトウ", "Ito"), 203357),
            (("渡辺", "ワタナベ", "Watanabe"), 200504),
            (("山本", "ヤマモト", "Yamamoto"), 200134),
            (("中村", "ナカムラ", "Nakamura"), 195219),
            (("小林", "コバヤシ", "Kobayashi"), 191819),
            (("加藤", "カトウ", "Kato"), 160283),
            (("吉田", "ヨシダ", "Yoshida"), 154461),
            (("山田", "ヤマダ", "Yamada"), 151675),
            (("佐々木", "ササキ", "Sasaki"), 135927),
            (("山口", "ヤマグチ", "Yamaguchi"), 119501),
            (("松本", "マツモト", "Matsumoto"), 116490),
            (("井上", "イノウエ", "Inoue"), 111287),
            (("木村", "キムラ", "Kimura"), 107446),
            (("林", "ハヤシ", "Hayashi"), 101826),
            (("斎藤", "サイトウ", "Saito"), 101774),
            (("清水", "シミズ", "Shimizu"), 97826),
            (("山崎", "ヤマザキ", "Yamazaki"), 90781),
            (("阿部", "アベ", "Abe"), 86833),
            (("森", "モリ", "Mori"), 86507),
            (("池田", "イケダ", "Ikeda"), 84860),
            (("橋本", "ハシモト", "Hashimoto"), 82836),
            (("山下", "ヤマシタ", "Yamashita"), 80588),
            (("石川", "イシカワ", "Ishikawa"), 77471),
            (("中島", "ナカジマ", "Nakajima"), 74106),
            (("前田", "マエダ", "Maeda"), 72930),
            (("藤田", "フジタ", "Fujita"), 72375),
            (("後藤", "ゴトウ", "Goto"), 71629),
            (("小川", "オガワ", "Ogawa"), 71179),
            (("岡田", "オカダ", "Okada"), 70347),
            (("長谷川", "ハセガワ", "Hasegawa"), 69201),
            (("村上", "ムラカミ", "Murakami"), 68606),
            (("近藤", "コンドウ", "Kondo"), 68297),
            (("石井", "イシイ", "Ishii"), 67079),
            (("遠藤", "エンドウ", "Endo"), 62620),
            (("斉藤", "サイトウ", "Saito"), 62540),
            (("坂本", "サカモト", "Sakamoto"), 62308),
            (("青木", "アオキ", "Aoki"), 59516),
            (("藤井", "フジイ", "Fujii"), 59204),
            (("西村", "ニシムラ", "Nishimura"), 58821),
            (("福田", "フクダ", "Fukuda"), 58714),
            (("太田", "オオタ", "Ota"), 58439),
            (("三浦", "ミウラ", "Miura"), 58006),
            (("藤原", "フジワラ", "Fujiwara"), 57742),
            (("松田", "マツダ", "Matsuda"), 55883),
            (("岡本", "オカモト", "Okamoto"), 55539),
            (("中川", "ナカガワ", "Nakagawa"), 55221),
        )
    )

    # for backwards compatibility only. use the pairs instead
    last_names = tuple(map(itemgetter(0), last_name_pairs))
    last_kana_names = tuple(map(itemgetter(1), last_name_pairs))
    last_romanized_names = tuple(map(itemgetter(2), last_name_pairs))

    formats_male = ("{{last_name}} {{first_name_male}}",)

    formats_female = ("{{last_name}} {{first_name_female}}",)

    formats = formats_male + formats_female

    kana_formats_male = ("{{last_kana_name}} {{first_kana_name_male}}",)

    kana_formats_female = ("{{last_kana_name}} {{first_kana_name_female}}",)

    kana_formats = kana_formats_male + kana_formats_female

    romanized_formats_male = ("{{first_romanized_name_male}} {{last_romanized_name}}",)

    romanized_formats_female = ("{{first_romanized_name_female}} {{last_romanized_name}}",)

    romanized_formats = romanized_formats_male + romanized_formats_female

    def first_name_pair(self) -> Tuple[str, str, str]:
        """
        @example ('明美', 'アケミ', 'Akemi')
        """
        return self.random_element(self.first_name_pairs)

    def first_name_male_pair(self) -> Tuple[str, str, str]:
        """
        @example ('晃', 'アキラ', 'Akira')
        """
        return self.random_element(self.first_name_male_pairs)

    def first_name_female_pair(self) -> Tuple[str, str, str]:
        """
        @example ('明美', 'アケミ', 'Akemi')
        """
        return self.random_element(self.first_name_female_pairs)

    def last_name_pair(self) -> Tuple[str, str, str]:
        """
        @example ('佐藤', 'サトウ', 'Sato')
        """
        return self.random_element(self.last_name_pairs)

    def first_name(self) -> str:
        """
        @example '明美'
        """
        return self.first_name_pair()[0]

    def first_name_male(self) -> str:
        """
        @example '晃'
        """
        return self.first_name_male_pair()[0]

    def first_name_female(self) -> str:
        """
        @example '明美'
        """
        return self.first_name_female_pair()[0]

    def last_name(self) -> str:
        """
        @example '佐藤'
        """
        return self.last_name_pair()[0]

    def first_kana_name(self) -> str:
        """
        @example 'アケミ'
        """
        return self.first_name_pair()[1]

    def first_kana_name_male(self) -> str:
        """
        @example 'アキラ'
        """
        return self.first_name_male_pair()[1]

    def first_kana_name_female(self) -> str:
        """
        @example 'アケミ'
        """
        return self.first_name_female_pair()[1]

    def last_kana_name(self) -> str:
        """
        @example 'サトウ'
        """
        return self.last_name_pair()[1]

    def first_romanized_name(self) -> str:
        """
        @example 'Akemi'
        """
        return self.first_name_pair()[2]

    def first_romanized_name_male(self) -> str:
        """
        @example 'Akira'
        """
        return self.first_name_male_pair()[2]

    def first_romanized_name_female(self) -> str:
        """
        @example 'Akemi'
        """
        return self.first_name_female_pair()[2]

    def last_romanized_name(self) -> str:
        """
        @example 'Sato'
        """
        return self.last_name_pair()[2]

    def kana_name(self) -> str:
        """
        @example 'サトウ アケミ'
        """
        pattern: str = self.random_element(self.kana_formats)
        return self.generator.parse(pattern)

    def kana_name_male(self) -> str:
        """
        @example 'サトウ アキラ'
        """
        pattern: str = self.random_element(self.kana_formats_male)
        return self.generator.parse(pattern)

    def kana_name_female(self) -> str:
        """
        @example 'サトウ アケミ'
        """
        pattern: str = self.random_element(self.kana_formats_female)
        return self.generator.parse(pattern)

    def romanized_name(self) -> str:
        """
        @example 'Akemi Sato'
        """
        pattern: str = self.random_element(self.romanized_formats)
        return self.generator.parse(pattern)

    def romanized_name_male(self) -> str:
        """
        @example 'Akira Sato'
        """
        pattern: str = self.random_element(self.romanized_formats_male)
        return self.generator.parse(pattern)

    def romanized_name_female(self) -> str:
        """
        @example 'Akemi Sato'
        """
        pattern: str = self.random_element(self.romanized_formats_female)
        return self.generator.parse(pattern)
